#!/usr/bin/env python
# encoding: utf-8
"""
Generates a taget decoy database with consitent shuffled peptides
or reverse proteins

usage:
./generate_target_decoy_1_0_0.py <fastaDB1 fastaDB2 fastaDB3 ... fastaDBn>



Copyright 2013
by
Johannes Barth
Daniel Jaeger
Christian Fufezan
Stefan Schulze

"""

from __future__ import print_function
import sys
import random
import itertools
from collections import defaultdict as ddict
import ursgal
import re


def main(
    input_files=None,
    output_file=None,
    enzyme=None,
    decoy_tag='decoy_',
    mode='shuffle_peptide',
    convert_aa_in_motif=None,  # 'new_aa,motif,position_to_be_replaced'
):

    # first do the redundancy check of aa seqeucnes...
    sequenceFastaDict = ddict(list)
    print("Checking for redundancy of protein sequences...", file=sys.stderr)
    for fastaFile in input_files:
        for fastaID, sequence in ursgal.ucore.parse_fasta(open(fastaFile, "r")):
            if 'REVERSED' in fastaID:
                continue
            sequenceFastaDict[sequence].append(fastaID)
    print("Done!", file=sys.stderr)

    perDict = {}
    fastaDict = {}
    goodLookUp = {}
    unmutableTPSet = set()  # has to be on pep level since we add some later during ranking
    mutableTPSet = set()

    def generateMergedFastaKey(IDList):
        """Format the fastaID in the right way """
        if len(IDList) > 1:
            key = '{0}_NumberOfIdenticalSequences_{1}'.format(
                IDList[0],
                len(IDList)
            )
        else:
            key = '{0}'.format(IDList[0])
        return key

    def generateConvertedSequence(sequence, convert_aa_in_motif):
        '''
        Check if sequence has motif and if yes, replace specified aminoacid.
        
        Returns:
            str: new/converted sequence
        '''
        new_aa, motif, position = convert_aa_in_motif.split(',')
        matches = []
        for match in re.finditer(motif, sequence):
            matches.append(match.span())
        new_sequence = ''
        prev_end = 0
        for start, end in matches:
            new_sequence += sequence[prev_end:start]
            new_sequence += sequence[start:start + int(position)] + new_aa +\
                sequence[start + int(position) + 1:end]
            prev_end = end
        new_sequence += sequence[prev_end:len(sequence)]
        return new_sequence

    uniqueModels = list()
    nonUniqueModels = list()
    listNonUniqueModels = list()

    final_output = {}

    cleavage_aa, site, inhibitor = enzyme.split(';')

    counter = 0
    for sequence, fastaIDList in sequenceFastaDict.items():
        fastaID = generateMergedFastaKey(fastaIDList)
        if len(fastaIDList) > 1:
            nonUniqueModels.append(fastaID)
            listNonUniqueModels += fastaIDList
        else:
            uniqueModels.append(fastaID)
        #print(fastaID,file = sys.stderr )
        counter += 1
        if counter % 100 == 0:
            print(
                'Processing sequence #{0}/{1}'.format(
                    counter,
                    len(sequenceFastaDict.keys())
                ),
                end='\r',
                file=sys.stderr
            )
        
        if convert_aa_in_motif is not None:
            sequence = generateConvertedSequence(sequence, convert_aa_in_motif)

        if mode == 'reverse_protein':
            if sequence[-1] == '*':
                sequence_without_terminator = sequence[:-1]
            else:
                sequence_without_terminator = sequence

            final_output[fastaID] = {
                'new_sequence_target': '{0}'.format(
                    sequence_without_terminator
                ),
                'new_sequence_decoy' : '{0}'.format(
                    sequence_without_terminator[::-1]
                )
            }
        elif mode == 'shuffle_peptide':
            peptideList = ursgal.ucore.digest(
                sequence,
                (cleavage_aa, site),
                no_missed_cleavages=True
            )
            fastaDict[fastaID] = {
                'seqLen'                   : len(sequence),
                'peptides'                 : peptideList,
                'lengthOfDecoyablePetides' : 1
            }

            for n, peptide in enumerate(peptideList):
                # print(peptide)
                character_to_preserve_C = ''
                character_to_preserve_N = ''
                terminal_peptide_N = False
                terminal_peptide_C = False
                if n == 0 :
                    terminal_peptide_N = True
                if n == len(peptideList) - 1:
                    terminal_peptide_C = True
                if site == 'C':
                    if peptide[-1] in cleavage_aa:
                        character_to_preserve_C = peptide[-1]
                        if peptide[0] in inhibitor and terminal_peptide_N is False:
                            character_to_preserve_N = peptide[0]
                            peptideWithoutCleavageAA = peptide[1 :-1]
                        else:
                            peptideWithoutCleavageAA = peptide[:-1]
                    else:
                        peptideWithoutCleavageAA = peptide
                elif site == 'N':
                    if peptide[0] in cleavage_aa:
                        character_to_preserve_N = peptide[0]
                        if peptide[-1] in inhibitor and terminal_peptide_C is False:
                            character_to_preserve_C = peptide[-1]
                            peptideWithoutCleavageAA = peptide[1 :-1]
                        else:
                            peptideWithoutCleavageAA = peptide[1:]
                    else:
                        peptideWithoutCleavageAA = peptide
                else:
                    print('''
                        No cleavage site for {0} defined.
                        Please add this information to uparams.py
                        (enzyme --> generate_target_decoy_style_1)
                        '''.format(enzyme))
                    sys.exit(1)
                aaString = ''.join(sorted(peptide))
                if aaString not in perDict.keys():
                    perDict[aaString] = {
                        'permutations'  : [],
                        'pepProtTuples' : set(),
                        'peptides'      : set(),
                        'mutable'       : True,
                        'permutated'    : False
                    }

                if len(peptide) < 8:
                    if perDict[aaString]['permutated'] is False:
                        for permutation in itertools.permutations(peptideWithoutCleavageAA, len(peptideWithoutCleavageAA)):
                            permutedSequence = '{0}{1}{2}'.format(
                                character_to_preserve_N,
                                ''.join(permutation),
                                character_to_preserve_C
                            )
                            check = check_decoy_sequence(
                                permuted_sequence=permutedSequence,
                                cleavage_site=site,
                                inhibitor=inhibitor,
                                terminal_peptide_N=terminal_peptide_N,
                                terminal_peptide_C=terminal_peptide_C,
                                preserved_C=character_to_preserve_C,
                                preserved_N=character_to_preserve_N,
                            )
                            if check is False:
                                continue
                            perDict[aaString]['permutations'].append(
                                permutedSequence)

                        perDict[aaString]['permutations'] = list(
                            set(perDict[aaString]['permutations'])
                        )
                        perDict[aaString]['permutated'] = True

                    if peptide in perDict[aaString]['permutations']:
                        perDict[aaString]['permutations'].remove(peptide)

                    perDict[aaString]['pepProtTuples'].add((peptide, fastaID))
                    perDict[aaString]['peptides'].add(peptide)
                else:
                    if perDict[aaString]['mutable']:
                        a = 50000  # math.factorial(len(list2shuffle)) * 10
                        list2shuffle = list(peptideWithoutCleavageAA)
                        while a:
                            random.shuffle(list2shuffle)

                            shuffled_peptide = '{0}{1}{2}'.format(
                                character_to_preserve_N,
                                ''.join(list2shuffle),
                                character_to_preserve_C
                            )

                            check = check_decoy_sequence(
                                permuted_sequence=shuffled_peptide,
                                cleavage_site=site,
                                inhibitor=inhibitor,
                                terminal_peptide_N=terminal_peptide_N,
                                terminal_peptide_C=terminal_peptide_C,
                                preserved_C=character_to_preserve_C,
                                preserved_N=character_to_preserve_N,
                            )

                            if shuffled_peptide != peptide and check is True:
                                if shuffled_peptide not in perDict[aaString]['permutations']:
                                    perDict[aaString]['permutations'].append(
                                        shuffled_peptide)
                                    perDict[aaString]['pepProtTuples'].add(
                                        (peptide, fastaID))
                                    perDict[aaString]['peptides'].add(peptide)
                                    break
                            a -= 1
                        if a == 0:
                            perDict[aaString]['mutable'] = False
                            unmutableTPSet.add(peptide)

        else:
            print('Mode {0} not specified'.format(mode))
            sys.exit(1)

    if mode == 'shuffle_peptide':
        for aaString in perDict.keys():
            if len(perDict[aaString]["permutations"]) >= len(perDict[aaString]["peptides"]):
                for peptide, fastaID in perDict[aaString]["pepProtTuples"]:
                    randomPos = random.randrange(
                        len(perDict[aaString]["permutations"]))
                    if peptide not in mutableTPSet:
                        goodLookUp[peptide] = perDict[
                            aaString]["permutations"][randomPos]
                    mutableTPSet.add(peptide)
                    fastaDict[fastaID][
                        'lengthOfDecoyablePetides'] += len(aaString)

            elif len(perDict[aaString]['permutations']) == 0:
                unmutableTPSet |= perDict[aaString]['peptides']

        for aaString in perDict.keys():
            if 0 < len(perDict[aaString]['permutations']) < len(perDict[aaString]['peptides']):
                tmp = sorted([(fastaDict[fastaID]['seqLen'] / fastaDict[fastaID]['lengthOfDecoyablePetides'] ,
                               fastaID, peptide) for peptide, fastaID in perDict[aaString]['pepProtTuples']])

                for n, (coverage, fastaID, peptide) in enumerate(tmp):
                    if n >= len(perDict[aaString]['permutations']):
                        unmutableTPSet.add(peptide)
                        #print('Forever alone :', peptide , file = sys.stderr )
                    else:
                        mutableTPSet.add(peptide)
                        goodLookUp[peptide] = perDict[
                            aaString]['permutations'][n]
                        fastaDict[fastaID][
                            'lengthOfDecoyablePetides'] += len(aaString)
        print(
            'Mutable: {0}, not mutable : {1}, i.e. {2:4.3f}%'.format(
                len(mutableTPSet),
                len(unmutableTPSet),
                100.0 * len(unmutableTPSet) / len(mutableTPSet)
            ),
            file=sys.stderr
        )
        c = ddict(int)
        counter = 0
        for sequence, fastaIDList in sequenceFastaDict.items():
            fastaID = generateMergedFastaKey(fastaIDList)
            counter += 1
            if counter % 100 == 0:
                print(
                    'Buffering new sequence #{0}/{1}'.format(
                        counter,
                        len(sequenceFastaDict)
                    ),
                    end="\r",
                    file=sys.stderr
                )

            if convert_aa_in_motif is not None:
                sequence = generateConvertedSequence(sequence, convert_aa_in_motif)

            peptideList = ursgal.ucore.digest(
                sequence,
                (cleavage_aa, site),
                no_missed_cleavages=True
            )
            originalSequence = []
            shuffledSequence = []
            for peptide in peptideList:
                originalSequence.append(peptide)
                if peptide not in goodLookUp.keys():
                    shuffledPeptide = peptide
                else:
                    shuffledPeptide = goodLookUp[peptide]
                shuffledSequence.append(shuffledPeptide)
                if peptide in unmutableTPSet:
                    c[peptide] += 1
            new_sequence_target = "".join(originalSequence)
            new_sequence_decoy = "".join(shuffledSequence)
            if len(new_sequence_decoy) == 0 or len(new_sequence_target) == 0:
                print(
                    "WARNING: FastaID: {0} has sequence with length zero".format(
                        fastaID
                    ),
                    file=sys.stderr
                )
                continue
            final_output[fastaID] = {
                'new_sequence_target' : new_sequence_target,
                'new_sequence_decoy'  : new_sequence_decoy
            }
        unmutable_peptides = output_file + '_unmutable_peptides.txt'
        print(
            'Writing unmutable peptides in {0}'.format(
                unmutable_peptides
            ),
            file=sys.stderr
        )

        with open(unmutable_peptides , 'w') as io:
            for v, k in sorted([(v, k) for k, v in c.items()], reverse=True):
                print(
                    '{0: >10}\t{1}'.format(
                        v,
                        k
                    ),
                    file=io
                )

    print(file=sys.stderr)
    # NOTE the reduncdancy printout:
    print(
        '=== Number of NON UNIQUE protein models with the same sequence ==='
    )
    print(
        '{0} These were merged to {1} new protein model names'.format(
            len(listNonUniqueModels),
            len(nonUniqueModels)
        ),
        file=sys.stderr
    )

    print(
        '=== Number of UNIQUE protein models ==='
    )
    print(
        '{0}'.format(
            len(uniqueModels)
        ),
        file=sys.stderr
    )

    opened_output_file = open(
        output_file,
        'w'
    )
    if sys.platform == 'win32':
        line_ending = '\n'
    else:
        line_ending = '\r\n'
    total_sequences_2_write = len(final_output.keys())
    for n, (fastaID, sequence_dict) in enumerate(final_output.items()):
        if n % 100 == 0:
            print(
                'Writing new sequence #{0}/{1}'.format(
                    n,
                    total_sequences_2_write
                ),
                end="\r",
                file=sys.stderr
            )
        new_sequence_target = sequence_dict['new_sequence_target']
        new_sequence_decoy = sequence_dict['new_sequence_decoy']
        # targets, just print the sequence
        print(">{0}".format(fastaID), file=opened_output_file, end=line_ending)
        for pos, _ in enumerate(new_sequence_target):
            print(_, end="", file=opened_output_file)
            if (pos + 1) % 80 == 0:
                print(file=opened_output_file, end=line_ending)
        print('', file=opened_output_file, end=line_ending)
        # decoy, print the shuffled peptides

        print(">{0}{1}".format(decoy_tag, fastaID),
              file=opened_output_file, end=line_ending)

        for pos, _ in enumerate(new_sequence_decoy):
            print(_, end="", file=opened_output_file)
            if (pos + 1) % 80 == 0:
                print(file=opened_output_file, end=line_ending)
        print('', file=opened_output_file, end=line_ending)

    print(file=sys.stderr)

    return output_file


def check_decoy_sequence(
    permuted_sequence=None,
    cleavage_site=None,
    inhibitor=None,
    terminal_peptide_N=False,
    terminal_peptide_C=False,
    preserved_C=None,
    preserved_N=None
):
    allowed_decoy = True
    if cleavage_site == 'C':
        if permuted_sequence[0] in inhibitor and terminal_peptide_N is False:
            if permuted_sequence[0] not in preserved_N:
                allowed_decoy = False
    elif cleavage_site == 'N':
        if permuted_sequence[-1] in inhibitor and terminal_peptide_C is False:
            if permuted_sequence[-1] not in preserved_C:
                allowed_decoy = False
    return allowed_decoy

if __name__ == '__main__':
    if len(sys.argv) == 1:
        print(__doc__)
    else:
        output_file = 'BSA_target_decoy_test.fasta'
        name_of_db = main(
            sys.argv[1:],
            output_file,
            enzyme=('RK;C;P'),
            mode='shuffle_peptide'
        )
        print(name_of_db)



#!/usr/bin/env python3
# encoding: utf-8
'''

Test the unify_csv function for msfragger engine

Covers 
    * Reassignment of merged 15N + e.g. Oxidation modifications back to normal
      Oxidation and removement of any 

'''
import ursgal
import csv
import pickle
import os

modifications = [
    '*,opt,Prot-N-term,Acetyl',
    'M,opt,any,Oxidation',
    'C,fix,any,Carbamidomethyl',  # Carbamidomethylation
]

R = ursgal.UController(
    params = {
        'modifications' : modifications
    }
)
R.map_mods()

scan_rt_lookup = pickle.load(
    open(
        os.path.join(
            'tests',
            'data',
            '_test_ursgal_lookup.pkl')
        ,
        'rb'
    )
)

unify_csv_main = R.unodes['unify_csv_1_0_0']['class'].import_engine_as_python_function()
input_csv = os.path.join(
    'tests',
    'data',
    'msfragger_20170103',
    'test_BSA1_msfragger_20170103.csv'
)
output_csv = os.path.join(
    'tests',
    'data',
    'msfragger_20170103',
    'test_BSA1_msfragger_20170103_unified.csv'
)
unify_csv_main(
    input_file     = input_csv,
    output_file    = output_csv,
    scan_rt_lookup = scan_rt_lookup,
    params = {
        'translations' : {
            'decoy_tag' : 'decoy_',
            'enzyme'    : 'KR;C;P',
            'semi_enzyme' : False,
            'database' : os.path.join(
                'tests',
                'data',
                'BSA.fasta'
            ),
            'protein_delimiter'              : '<|>',
            'psm_merge_delimiter'            : ';',
            'keep_asp_pro_broken_peps'       : True,
            'precursor_mass_tolerance_minus' : 5,
            'precursor_mass_tolerance_plus'  : 5,
            'precursor_isotope_range'        : "0,1",
            'max_missed_cleavages'           : 2,
            'rounded_mass_decimals' : 3,
            'use_pyqms_for_mz_calculation' : False,
            'aa_exception_dict' : {
                'J' : {
                    'original_aa' : ['L','I'],
                },
                'O' : {
                    'original_aa' : ['K'],
                    'unimod_name' : 'Methylpyrroline',
                },
            },
        },
        'label' : '15N',
        'mods' : R.params['mods']
    },
    search_engine  = 'msfragger_20170103',
)

ident_list = [ ]
for line_dict in csv.DictReader(open(output_csv, 'r')):
    ident_list.append( line_dict )


def unify_msfragger_test():
    assert len(ident_list) == 3, '''
    Unexpected number of lines in output csv.
    Probably lines were merged that should not be merged.
    '''
    for test_id, test_dict in enumerate(ident_list):
        yield unify_msfragger, test_dict


def unify_msfragger( test_dict ):
    assert 'uCalc m/z' in test_dict.keys()

    for key in [
            'Retention Time (s)',
            'Spectrum ID',
            'Modifications',
            'Spectrum Title',
            'Sequence',
            'MSFragger:Neutral mass of peptide',
        ]:
        test_value = test_dict[key]
        expected_value = test_dict['Expected {0}'.format(key)]
        if key == 'Retention Time (s)':
            test_value     = round(float(test_value), 4)
            expected_value = round(float(expected_value), 4)

        assert test_value == expected_value, '''
Unexpected value in column "{0}":
    test value:     {1}
    expected value: {2}
            '''.format(
                key,
                test_value,
                expected_value
            )


if __name__ == '__main__':
    print(__doc__)
    # for test_id, test_dict in enumerate(ident_list):
    #     unify_msfragger(test_dict)



#!/usr/bin/env python
import ursgal
import importlib
import os
import sys
import pickle


class unify_csv_1_0_0( ursgal.UNode ):
    """unify_csv_1_0_0 UNode

    Unifies the .csv files of converted search engine results.
    The corrections for each engine are listed in the node under
    ursgal/resources/platform_independent/arc_independent/unify_csv_1_0_0
    """
    META_INFO = {
        'edit_version'       : 1.00,
        'name'               : 'Unify CSV',
        'version'            : '1.0.0',
        'release_date'       : None,
        'engine_type' : {
            'converter' : True
        },
        'input_extensions'   : ['.csv'],
        'output_extensions'  : ['.csv'],
        'output_suffix'      : 'unified',
        'include_in_git'     : True,
        'in_development'     : True,
        'distributable'      : True,
        'utranslation_style' : 'unify_csv_style_1',
        'engine' : {
            'platform_independent' : {
                'arc_independent' : {
                    'exe' : 'unify_csv_1_0_0.py',
                },
            },
        },
        'citation' : \
            'Kremer, L. P. M., Leufken, J., Oyunchimeg, P., Schulze, S. & '\
            'Fufezan, C. (2016) Ursgal, Universal Python Module Combining '\
            'Common Bottom-Up Proteomics Tools for Large-Scale Analysis. J. '\
            'Proteome res. 15, 788-794.',
    }

    def __init__(self, *args, **kwargs):
        super(unify_csv_1_0_0, self).__init__(*args, **kwargs)

    def _execute( self ):
        '''
        Result files from search engines are unified
        to contain the same informations in the same style

        Input file has to be a .csv

        Creates a _unified.csv file and returns its path

        '''
        print('[ -ENGINE- ] Executing conversion ..')
        unify_csv_main = self.import_engine_as_python_function()
        if self.params['output_file'].lower().endswith('.csv') is False:
            raise ValueError('Trying to unify a non-csv file')

        output_file = os.path.join(
            self.params['output_dir_path'],
            self.params['output_file']
        )
        input_file  = os.path.join(
            self.params['input_dir_path'],
            self.params['input_file']
        )

        scan_rt_lookup_path = self.meta_unodes['ucontroller'].scan_rt_lookup_path
        assert os.path.isfile( scan_rt_lookup_path ), """
Could not load RT lookup dict from this location: {0}
        """.format( scan_rt_lookup_path )

        scan_rt_lookup_dict = pickle.load(
            open( scan_rt_lookup_path, 'rb' )
        )

        # find the last search/denovo engine:
        last_engine = self.get_last_search_engine(
            history = self.stats['history'],
        )

        last_search_engine_colname = self.UNODE_UPARAMS['validation_score_field']['uvalue_style_translation'][last_engine]
        tmp_files = unify_csv_main(
            input_file      = input_file,
            output_file     = output_file,
            scan_rt_lookup  = scan_rt_lookup_dict,
            params          = self.params,
            search_engine   = last_engine,
            score_colname   = last_search_engine_colname,
        )
        for tmp_file in tmp_files:
            self.created_tmp_files.append(tmp_file)
        print()
        return output_file



#!/usr/bin/env python
import ursgal
import pprint
import csv
import os
import sys

FLOAT_FORMAT_STRING = '{0:1.1e}'


class qvality_2_02( ursgal.UNode ):
    """
    qvality_2_02 UNode

    q-value and posterior error probability calculation from score distributions

    Reference:
    Kﾃ､ll L, Storey JD, Noble WS (2009) QVALITY: non-parametric estimation of q-values and posterior error probabilities.
    """

    META_INFO = {
        'edit_version'       : 1.00,
        'name'               : 'QVALITY',
        'version'            : '1.0.0',
        'release_date'       : '2009-4-1',
        'engine_type' : {
            'validation_engine' : True,
        },
        'input_extensions'   : ['.csv'],
        'output_extensions'  : ['.csv'],
        'output_suffix'      : 'qvality_2_02_validated',
        'create_own_folder'  : False,
        'include_in_git'     : False,
        'group_psms'         : True,
        'in_development'     : False,
        'distributable'      : True,
        'utranslation_style' : 'qvality_style_1',
        'engine' : {
            'darwin' : {
                '64bit' : {
                    'exe'            : 'qvality',
                    'url'            : '',
                    'zip_md5'        : 'e578317394afff0cdbc121e0203d128b',
                    'additional_exe' : [],
                },
            },
            'linux' : {
                '64bit' : {
                    'exe'            : 'qvality',
                    'url'            : '',
                    'zip_md5'        : '50455452da7fb4124a6a433ed198c10d',
                    'additional_exe' : [],
                },
            },
            'win32' : {
                '64bit' : {
                    'exe'            : 'qvality.exe',
                    'url'            : '',
                    'zip_md5'        : '2b6bb4edc54b5712e61edb44baba41d5',
                    'additional_exe' : [],
                },
                # '32bit' : {
                #     'exe'            : 'qvality.exe',
                #     'url'            : '',
                #     'zip_md5'        : 'f10aee7feec1340364c64eccc6f75a3c',
                #     'additional_exe' : [],
                # },
            },
        },
        'citation' : \
            'Kall L, Storey JD, Noble WS (2009) QVALITY: non-parametric '\
            'estimation of q-values and posterior error probabilities.',
    }

    def __init__(self, *args, **kwargs):
        super(qvality_2_02, self).__init__(*args, **kwargs)
        pass

    def preflight( self ):
        '''
        Formating the command line to via self.params
        '''
        self.params['last_engine'] = self.get_last_search_engine( history = self.stats['history'] )

        translations = self.params['translations']['_grouped_by_translated_key']

        self.params['translations']['output_file_incl_path'] = os.path.join(
            self.params['output_dir_path'],
            self.params['output_file']
        )
        translations['-o']['output_file_incl_path'] = self.params['translations']['output_file_incl_path']

        self._generate_qvality_input_files()

        self.created_tmp_files += [
            self.params['translations']['target']['path'],
            self.params['translations']['decoy']['path'],
        ]

        self.params['command_list'] =[
            self.exe,
        ]

        for translated_key, translation_dict in translations.items():
            if list(translation_dict.values())[0] == None:
                continue
            elif translated_key in ['validation_minimum_score', 'validation_score_field', 'decoy_tag', '-r']:
                continue
            elif len(translation_dict) == 1:
                self.params['command_list'].extend((translated_key, str(list(translation_dict.values())[0])))
            else:
                print('The translatd key ', translated_key, ' maps on more than one ukey, but no special rules have been defined')
                print(translation_dict)
                sys.exit(1)

        if self.UNODE_UPARAMS['bigger_scores_better']['uvalue_style_translation'][self.params['last_engine']] is False:
            self.params['command_list'].append('-r') #False: lower scores are better e.g.OMSSA, scores have to be reversed for qvality
        self.params['command_list'] += [
            self.params['translations']['target']['path'],
            self.params['translations']['decoy']['path'],
        ]

    def postflight( self ):
        '''
        Parse the qvality output and merge it back into the csv file
        '''
        score_2_pep_and_qvalue_lookup = {}
        with open( self.params['translations']['output_file_incl_path'], 'r' ) as qvio:
            qvio.readline() # header gone ...
            for line in qvio:
                if line.strip() != '':
                    score, pep, qvalue = line.strip().split()
                    # if float(pep) >= self.params['maximum_pep_for_ident_csv']:
                    #     continue
                    formatted_score = FLOAT_FORMAT_STRING.format( float(score) )
                    if formatted_score not in score_2_pep_and_qvalue_lookup.keys():
                        score_2_pep_and_qvalue_lookup[ formatted_score ] = ( pep, qvalue )
                    else:
                        pass
        qvio.close()
        opened_file = open(
            os.path.join(
                self.params['input_dir_path'],
                self.params[ 'input_file' ]
            )
            , 'r' )
        csv_kwargs = {}
        if sys.platform == 'win32':
            csv_kwargs['lineterminator'] = '\n'
        else:
            csv_kwargs['lineterminator'] = '\r\n'

        csv_input = csv.DictReader( row for row in opened_file if not row.startswith('#') )

        csv_output = csv.DictWriter(
            open( self.params['translations']['output_file_incl_path'], 'w' ),
            csv_input.fieldnames + ['PEP','q-value'],
            **csv_kwargs
        )

        csv_output.writeheader()
        for line_dict in csv_input:
            #we skip all decoys ... not anymore, please use the filter functon instead
            # if line_dict['Is decoy'].upper() == 'TRUE':
            #     continue
            # if self.params['decoy_tag'] in line_dict['proteinacc_start_stop_pre_post_;']:
            #     continue
            formatted_score = FLOAT_FORMAT_STRING.format(
                float(
                    line_dict[
                        self.UNODE_UPARAMS['validation_score_field']['uvalue_style_translation'][self.params['last_engine']]
                    ]
                )
            )
            if formatted_score in score_2_pep_and_qvalue_lookup.keys():
                pep, qvalue              = score_2_pep_and_qvalue_lookup[ formatted_score ]
                line_dict['PEP']         = pep
                line_dict['q-value']     = qvalue
                csv_output.writerow( line_dict )
            else:
                pass
                # this code should sover the cases when a score is not found in the
                # lookup but was the best score in the spectrum...
                # use for debugging
                # psm_list = self.params['grouped_psms'][line_dict['Spectrum Title']]
                # for pos, (score_2_compare,psm_dict) in enumerate(psm_list):
                #     formatted_score_2_compare = FLOAT_FORMAT_STRING.format(
                #         score_2_compare
                #     )
                #     if formatted_score == formatted_score_2_compare and pos == 0:
                #         # some checking should be done, at least for debugging
                #         print('WARNING - qvality output scores could not be mapped and score was the best for spectrum!')
                #         print('Scores: ', formatted_score, formatted_score_2_compare )
                #         pprint.pprint( psm_list )
                #         sys.exit(1)



    def _generate_qvality_input_files( self ):
        '''
        creates target and decoy score txt files for qvality
        '''
        crazy_tmp_files = {}
        for tag in [ 'target', 'decoy' ]:
            self.params['translations'][tag] = {}
            self.params['translations'][tag]['path'] = '{0}_{1}_scores.txt'.format(
                os.path.join(
                    self.params['output_dir_path'],
                    self.params['file_root']
                    ),
                tag,
            )
            crazy_tmp_files[ tag ] = open(self.params['translations'][tag]['path'], 'w')
        defline_key = 'Protein ID'
        for spectrum_title, grouped_psm_list in self.params['grouped_psms'].items():
            score_2_write, psm_dict = grouped_psm_list[0]
            if psm_dict[ 'Is decoy'].upper() == 'TRUE':
                tag = 'decoy'
            else:
                tag = 'target'

            min_score = self.UNODE_UPARAMS['validation_minimum_score']['uvalue_style_translation'][self.params['last_engine']]
            if score_2_write < min_score:
                score_2_write = min_score
            print(
                score_2_write,
                file = crazy_tmp_files[ tag ]
            )
        for tag in [ 'target', 'decoy' ]:
            crazy_tmp_files[ tag ].close()



#!/usr/bin/env python3.3
from .mzidentml_lib_1_6_10 import mzidentml_lib_1_6_10 as ml


class mzidentml_lib_1_7( ml ):
    '''
    MzidLib 1_7UNode

    Import functions from mzidentml_lib_1_6_10

    Note:

        Please download and install manually from http://www.proteoannotator.org/?q=installation

    '''
    META_INFO = {
        'edit_version'       : 1.00,
        'name'               : 'MzidLib',
        'version'            : '1.7',
        'release_date'       : None,
        'engine_type' : {
            'converter' : True
        },
        'input_extensions'   : ['.xml', '.xml.gz', '.csv', '.mzid', '.mzid.gz'],
        'output_extensions'  : ['.csv'],
        'in_development'     : False,
        'output_suffix'      : None,
        'include_in_git'     : None,
        'distributable'      : False,
        'utranslation_style' : 'mzidentml_style_1',
        'engine' : {
            'platform_independent' : {
                'arc_independent' : {
                    'exe'            : 'mzidlib-1.7.jar',
                    'url'            : 'http://www.proteoannotator.org/?q=installation',
                    'zip_md5'        : '',
                    'additional_exe' : [],
                },
            },
        },
        'citation' : \
            'Reisinger F, Krishna R, Ghali F, Rios D, Hermjakob H, '\
            'Vizcaino JA, Jones AR. (2012) jmzIdentML API: A Java interface '\
            'to the mzIdentML standard for peptide and protein identification '\
            'data.',
    }
    pass



#!/usr/bin/env python3
# encoding: utf-8

import ursgal
import os


def main():
    '''
    Examples script for filtering unified results for modification containing
    entries

    usage:
        ./filter_csv_for_mods_example.py


    Will produce a file with only entries which contain Carbamidomethyl as a
    modification.
    '''
    params = {
        'csv_filter_rules': [
            ['Modifications', 'contains', 'Carbamidomethyl'],
        ],
        'write_unfiltered_results': False

    }

    csv_file_to_filter = os.path.join(
        os.pardir,
        'example_data',
        'misc',
        'filter_csv_for_mods_example_omssa_2_1_9_pmap_unified.csv'
    )
    uc = ursgal.UController(
        params=params
    )

    filtered_csv = uc.execute_misc_engine(
        input_file=csv_file_to_filter,
        engine='filter_csv_1_0_0',
    )


if __name__ == '__main__':
    main()



#!/usr/bin/env python
import ursgal


class comet_2014_02_rev_2( ursgal.UNode ):
    """
    comet_2014_02_rev_2 UNode

    Not implemented yet
    """
    META_INFO = {
        'edit_version'      : 1.00,
        'name'              : 'Comet',
        'version'           : '2014_02_rev_2',
        'release_date'      : '2014-1-10',
        'engine_type' : {
            'protein_database_search_engine' : True,
        },
        'input_extensions'  : [],
        'output_extensions' : [],
        'in_development'    : True,
        'include_in_git'    : None,
        'distributable'      : False,
        'utranslation_style' : 'comet_style_1',
        'citation' : \
            '',
    }

    def __init__(self, *args, **kwargs):
        super(comet_2014_02_rev_2, self).__init__(*args, **kwargs)

    def preflight( self ):
        self.params[ 'command_list' ] = [
            'java', '-Xmx{java_-Xmx}'.format( **self.params),
            '-jar', '{executable_path}'.format(**self.params),
        ]
        self.params[ 'command_list' ] += self.params['options']

    def postflight( self ):
        pass



#!/usr/bin/env python
import ursgal
import os
import csv
import subprocess
import sys
import copy

class msamanda_2_0_0_9695( ursgal.UNode ):
    """
    MSAmanda 2_0_0_9695 UNode
    Parameter options at http://ms.imp.ac.at/inc/pd-nodes/msamanda/Manual%20MS%20Amanda%20Standalone.pdf

    Note:
    Please download and install MSAmanda manually from
    http://ms.imp.ac.at/?goto=msamanda

    Reference:
    Dorfer V, Pichler P, Stranzl T, Stadlmann J, Taus T, Winkler S, Mechtler K. (2014) MS Amanda, a universal identification algorithm optimized for high accuracy tandem mass spectra.

    """
    META_INFO = {
        'edit_version'       : 1.00,
        'name'               : 'MSAmanda',
        'version'            : '2.0.0.9695',
        'release_date'       : None,
        'engine_type' : {
            'protein_database_search_engine' : True,
        },
        'input_extensions'   : ['.mgf'],
        'output_extensions'  : ['.csv'],
        'create_own_folder'  : True,
        'include_in_git'     : False,
        'distributable'  : False,
        'in_development'     : False,
        'utranslation_style' : 'msamanda_style_1',
        'engine' : {
            'win32' : {
                '64bit' : {
                    'exe'            : 'MSAmanda.exe',
                    'url'            : '',
                    'zip_md5'        : None,
                    'additional_exe' : [],
                },
            },
        },
        'citation' : \
            'Dorfer V, Pichler P, Stranzl T, Stadlmann J, Taus T, Winkler S, '\
            'Mechtler K. (2014) MS Amanda, a universal identification '\
            'algorithm optimized for high accuracy tandem mass spectra.',
    }

    def __init__(self, *args, **kwargs):
        super(msamanda_2_0_0_9695, self).__init__(*args, **kwargs)
        if sys.platform in ['win32']:
            self.dependencies_ok = True
        else:
            try:
                proc = subprocess.Popen( ['mono', '-V'], stdout = subprocess.PIPE)
            except FileNotFoundError:
                print( '''
        ERROR: MS Amanda requires Mono 3.10.0 or newer.
        Installation: http://www.mono-project.com/download'''
                )

                self.dependencies_ok = False
        pass

    def preflight( self ):
        '''
        Formatting the command line via self.params

        Settings file is created in the output folder
        and added to self.created_tmp_files (can be deleted)

        Returns:
                self.params(dict)
        '''

        translations = self.params['translations']['_grouped_by_translated_key']

        self.params['translations']['mgf_input_file'] = os.path.join(
            self.params['input_dir_path'],
            self.params['input_file']
        )

        self.params['translations']['output_file_incl_path'] = os.path.join(
            self.params['output_dir_path'],
            self.params['output_file']
        )

        # if translations['unimod_file_incl_path']['unimod_file_incl_path'] == '' :
        self.params['translations']['unimod_file_incl_path'] = os.path.join(
            os.path.dirname(__file__),
            '..',
            'resources',
            'platform_independent',
            'arc_independent',
            'ext',
            'unimod.xml'
        )

        # building command_list !
        #
        if sys.platform in ['win32']:
            self.params['command_list'] = []
        else:
            self.params['command_list'] = ['mono']
        self.params['command_list'] += [
            self.exe,
            '{mgf_input_file}'.format(**self.params['translations']),
            '{database}'.format(**self.params['translations']),
            '{0}'.format( self.params['translations']['output_file_incl_path'] + '_settings.xml' ),
            '{output_file_incl_path}'.format(**self.params['translations'])
        ]
        self.created_tmp_files.append(self.params['translations']['output_file_incl_path'] + '_settings_1.xml')

        score_ions = []
        instruments_file_input = []
        for ion in [ "a", "b", "c", "x", "y", "z", "-H2O", "-NH3", "Imm", "z+1", "z+2", "INT" ]:
            if ion.lower() in self.params['translations']['score_ion_list']:
                score_ions.append( ion )
                instruments_file_input.append('''<series>{0}</series>'''.format(ion))
        instruments_file_input.append('''</setting>''')
        instruments_file_input.append('''</instruments>''')
        self.params['translations']['score_ions'] = ', '.join( score_ions )
        self.params['translations']['instruments_file_input'] = ''.join( instruments_file_input )

        print(
            '''
            [ WARNING ] precursor_mass_tolerance_plus and precursor_mass_tolerance_minus
            [ WARNING ] need to be combined for MS Amanda (use of symmetric tolerance window).
            [ WARNING ] The arithmetic mean is used.
            '''
        )
        self.params['translations']['precursor_mass_tolerance'] = (
            float(self.params['translations']['precursor_mass_tolerance_plus']) +
            float(self.params['translations']['precursor_mass_tolerance_minus'])
        ) / 2.0

        considered_charges = []
        for charge in range(
                int(self.params['translations'][ 'precursor_min_charge' ]),
                int(self.params['translations'][ 'precursor_max_charge' ]) + 1 
        ):
            considered_charges.append( '+{0}'.format(charge) )
        self.params['translations']['considered_charges'] = ', '.join( considered_charges )

        if self.params['translations']['label'] == '15N':
            for aminoacid, N15_Diff in ursgal.ukb.DICT_15N_DIFF.items():
                existing = False
                for mod in self.params[ 'mods' ][ 'fix' ]:
                    if aminoacid == mod[ 'aa' ]:
                        mod[ 'mass' ] += N15_Diff
                        mod[ 'name' ] += '_15N_{0}'.format(aminoacid)
                        existing = True
                if existing == True:
                    continue
                self.params[ 'mods' ][ 'fix' ].append(
                    { 
                        'pos'   : 'any',
                         'aa'   : aminoacid,
                         'name' : '15N_{0}'.format(aminoacid),
                         'mass' : N15_Diff 
                    }
                )
        self.params['translations']['enzyme_name'] = self.params['enzyme']
        self.params['translations']['enzyme_cleavage'], self.params['translations']['enzyme_position'], self.params['translations']['enzyme_inhibitors'] = self.params['translations']['enzyme'].split(';')
        self.params['translations']['enzyme'] = self.params['enzyme']

        modifications = [ ]
        for t in [ 'fix', 'opt' ]:
            fix = 'false'
            if t == 'fix':
                fix = 'true'
            for mod in self.params[ 'mods' ][ t ]:
                protein = 'false'
                n_term = 'false'
                c_term = 'false'
                if '>' in mod['name']:
                    print(
                        '''
                        [ WARNING ] MS Amanda cannot deal with '>'
                        [ WARNING ] in the modification name
                        [ WARNING ] Continue without modification {0} '''.format(mod, **mod)
                        )
                    continue
                if 'Prot' in mod[ 'pos' ]:
                    protein = 'true'
                if 'N-term' in mod[ 'pos' ]:
                    n_term = 'true'
                if 'C-term' in mod[ 'pos' ]:
                    c_term = 'true'
                if '*' in mod[ 'aa' ]:
                    modifications.append( '<modification fix="{0}" protein="{1}" nterm="{2}" cterm="{3}" delta_mass="{4}">{5}</modification>'.format(
                                        fix, protein, n_term, c_term, mod[ 'mass' ], mod[ 'name' ] ))
                    continue
                modifications.append( '<modification fix="{0}" protein="{1}" nterm="{2}" cterm="{3}" delta_mass="{4}">{5}({6})</modification>'.format(
                                    fix, protein, n_term, c_term, mod[ 'mass' ], mod['name'], mod[ 'aa' ] )
                                )

        self.params['translations']['modifications'] = ''.join( modifications )

        templates = self.format_templates( )
        for file_name, content in templates.items():
            file2write = self.params['translations']['output_file_incl_path'] + file_name
            with open(
                    file2write,
                    'w'
                ) as out:
                print(content, file=out)
                self.print_info(
                    'Wrote input file {0}'.format(
                        file2write
                    ),
                    caller = 'Info'
                )
                self.created_tmp_files.append(
                    file2write
                )
        return self.params

    def postflight( self ):
        '''
        Convert .tsv result files to .csv
        '''
        self.meta_unodes['ucontroller'].verify_engine_produced_an_output_file(
            self.params['translations']['output_file_incl_path'], 'MSAmanda'
        )

        cached_msamanada_output = []
        with open(self.params['translations']['output_file_incl_path'], 'r') as result_file:
            csv_dict_reader_object = csv.DictReader(
                (row for row in result_file if not row.startswith('#')),
                delimiter = '\t'
            )
            headers = csv_dict_reader_object.fieldnames
            translated_headers = []
            for header in headers:
                translated_headers.append(
                    self.UNODE_UPARAMS['header_translations']['uvalue_style_translation'].get(header, header)
                )
            translated_headers += [
                'Is decoy',
                'Start',
                'Stop',
                'Calc m/z'
            ]
            self.print_info(
                'Loading unformatted MS Amanda results ...',
                caller = 'Info'
            )
            for line_dict in csv_dict_reader_object:
                cached_msamanada_output.append( line_dict )
        self.print_info(
            'Loading unformatted MS Amanda results done',
            caller = 'postflight'
        )

        self.params['output_file'] = self.params['output_file'].replace(
            'tsv',
            'csv'
        )
        with open(
            os.path.join(
                self.params['output_dir_path'],
                self.params['output_file'],
            ),
            'w'
        ) as result_file:
            csv_dict_writer_object = csv.DictWriter(
                result_file,
                fieldnames = translated_headers
            )
            csv_dict_writer_object.writeheader()
            self.print_info(
                'Writing MS Amanda results, this can take a while...',
                caller = 'Info'
            )
            csv_write_list = []

            total_docs =len(cached_msamanada_output)

            for cache_pos, m in enumerate(cached_msamanada_output):
                tmp = {}
                for header in headers:
                    translated_header = self.UNODE_UPARAMS['header_translations']['uvalue_style_translation'].get(header, header)
                    tmp[ translated_header ] = m[ header ]
                tmp['Sequence'] = tmp['Sequence'].upper()

                if cache_pos % 500 == 0:
                    print(
                        '[ INFO ] Processing line number:    {0}/{1}'.format(
                            cache_pos,
                            total_docs
                        ),
                        end='\r'
                    )

                dict_2_write = copy.deepcopy( tmp )
                translated_mods = []
                #N-Term(Acetyl|42.010565|fixed);M1(Oxidation|15.994915|fixed);M23(Oxidation|15.994915|fixed)
                if dict_2_write['Modifications'] != '':
                    splitted_Modifications = dict_2_write['Modifications'].split(';')
                    for mod in splitted_Modifications:

                        position_or_aa_and_pos_unimod_name, mod_mass, fixed_or_opt = mod.split('|')
                        position_or_aa_and_pos,unimod_name = position_or_aa_and_pos_unimod_name.split('(')
                        position_or_aa_and_pos = position_or_aa_and_pos.strip()
                        unimod_name = unimod_name.strip()

                        if position_or_aa_and_pos.upper() == 'N-TERM':
                            position = 0
                        else:
                            position = position_or_aa_and_pos[ 1: ]

                        translated_mods.append(
                            '{0}:{1}'.format(
                                unimod_name,
                                position
                            )
                        )

                dict_2_write['Modifications'] = ';'.join( translated_mods )

                protein_id = tmp['proteinacc_start_stop_pre_post_;']

                csv_write_list.append( dict_2_write )
            duplicity_buffer = set()
            for final_dict_2_write in csv_write_list:
                duplicity_key = (
                    final_dict_2_write['Sequence'],
                    final_dict_2_write['Modifications'],
                    final_dict_2_write['proteinacc_start_stop_pre_post_;'],
                    final_dict_2_write['Spectrum ID']
                )
                if duplicity_key not in duplicity_buffer:
                    csv_dict_writer_object.writerow( final_dict_2_write )
                    duplicity_buffer.add( duplicity_key )
        self.print_info(
            'Writing MS Amanda results done!',
            caller = 'Info'
        )
        pass


    def format_templates( self ):
        self.params['translations']['exe_dir_path'] = os.path.dirname(self.exe)

        templates = {
            '_settings.xml' : '''<?xml version="1.0" encoding="utf-8" ?>
<settings>
    <search_settings>
        <enzyme specificity="{semi_enzyme}">{enzyme}</enzyme>
        <missed_cleavages>{max_missed_cleavages}</missed_cleavages>
        <modifications>
            {modifications}
        </modifications>
        <instrument>{score_ions}</instrument>
        <ms1_tol unit="{precursor_mass_tolerance_unit}">{precursor_mass_tolerance}</ms1_tol>
        <ms2_tol unit="{frag_mass_tolerance_unit}">{frag_mass_tolerance}</ms2_tol>
        <max_rank>{num_match_spec}</max_rank>
        <generate_decoy>{engine_internal_decoy_generation}</generate_decoy>
        <PerformDeisotoping>{deisotope_spec}</PerformDeisotoping>
        <MaxNoModifs>{max_num_per_mod}</MaxNoModifs>
        <MaxNoDynModifs>{max_num_mods}</MaxNoDynModifs>
        <MaxNumberModSites>{max_num_mod_sites}</MaxNumberModSites>
        <MaxNumberNeutralLoss>{max_num_neutral_loss}</MaxNumberNeutralLoss>
        <MaxNumberNeutralLossModifications>{max_num_neutral_loss_mod}</MaxNumberNeutralLossModifications>
        <MinimumPepLength>{min_pep_length}</MinimumPepLength>
    </search_settings>

  <basic_settings>
    <instruments_file>{output_file_incl_path}_instrument.xml</instruments_file>
    <unimod_file>{unimod_file_incl_path}</unimod_file>
    <enzyme_file>{output_file_incl_path}_enzymes.xml</enzyme_file>
    <monoisotopic>{precursor_mass_type}</monoisotopic>
    <considered_charges>{considered_charges}</considered_charges>
    <LoadedProteinsAtOnce>{batch_size}</LoadedProteinsAtOnce>
    <LoadedSpectraAtOnce>{batch_size_spectra}</LoadedSpectraAtOnce>
  </basic_settings>
</settings>
'''.format(**self.params['translations']),

        '_instrument.xml' : '''<?xml version="1.0"?>
<!-- possible values are "a", "b", "c", "x", "y", "z", "H2O", "NH3", "IMM", "z+1", "z+2", "INT" (for internal fragments) -->
<instruments>
  <setting name="{score_ions}">
{instruments_file_input}
'''.format(**self.params['translations']),

        '_enzymes.xml' : '''<?xml version="1.0" encoding="utf-8" ?>
<enzymes>
  <enzyme>
    <name>{enzyme_name}</name>
    <cleavage_sites>{enzyme_cleavage}</cleavage_sites>
    <inhibitors>{enzyme_inhibitors}</inhibitors>
    <position>{enzyme_position}</position>
  </enzyme>
</enzymes>'''.format(**self.params['translations']),
            }
        return templates



#!/usr/bin/env python
import ursgal
import os
import pprint
from collections import defaultdict as ddict
import csv
import itertools
import sys


class msfragger_20170103(ursgal.UNode):
    """
    MSFragger unode

    Note:
        Please download and install MSFragger manually from
        http://www.nesvilab.org/software.html

    Reference:
    Kong, A. T., Leprevost, F. V, Avtonomov, D. M., Mellacheruvu, D., and Nesvizhskii, A. I. (2017)
    MSFragger: ultrafast and comprehensive peptide identification in mass spectrometry–based
    proteomics. Nature Methods 14

    Note:
        Addition of user amino acids not implemented yet. Only mzML search
        possible at the moment. The mgf file can still be passed to the node,
        but the mzML has to be in the same folder as the mgf.

    Warning:
        Still in testing phase!
        Metabolic labeling based 15N search may still be errorprone. Use with
        care!

    """
    META_INFO = {
        'edit_version'                : 1.00,
        'name'                        : 'MSFragger',
        'version'                     : '20170103',
        'release_date'                : '2017-01-03',
        'utranslation_style'          : 'msfragger_style_1',
        'input_extensions'            : ['.mgf', '.mzML', '.mzXML'],
        'output_extensions'           : ['.csv'],
        'create_own_folder'           : True,
        'in_development'              : False,
        'include_in_git'              : False,
        'distributable'           : False,
        'engine_type' : {
            'protein_database_search_engine' : True,
        },
        'engine'                      : {
            'platform_independent'    : {
                'arc_independent' : {
                    'exe'            : 'MSFragger.jar',
                    'url'            : 'http://www.nesvilab.org/software.html',
                    'zip_md5'        : '',
                    'additional_exe' : [],
                },
            },
        },
        'citation'                   :
        'Kong, A. T., Leprevost, F. V, Avtonomov, '
            'D. M., Mellacheruvu, D., and Nesvizhskii, A. I. (2017) MSFragger: '
            'ultrafast and comprehensive peptide identification in mass '
            'spectrometry-based proteomics. Nature Methods 14'
    }

    def __init__(self, *args, **kwargs):
        super(msfragger_20170103, self).__init__(*args, **kwargs)
        pass

    def write_params_file(self):
        with open(self.param_file_name , 'w') as io:
            for msfragger_param_name, param_value in sorted(self.params_to_write.items()):
                print(
                    '{0} = {1}'.format(
                        msfragger_param_name,
                        param_value
                    ),
                    file=io
                )

        return

    def preflight(self):
        '''
        Formatting the command line and writing the param input file via 
        self.params

        Returns:
                dict: self.params
        '''
        self.param_file_name = os.path.join(
            self.params['output_dir_path'],
            'msfragger.params'
        )
        # further prepare and translate params

        # pprint.pprint(self.params['translations']['_grouped_by_translated_key'])
        # pprint.pprint(self.params)
        # exit()
        self.params_to_write = {
            'output_file_extension' : 'tsv',  # tsv or pepXML we fix it...
            'output_format'         : 'tsv',  # pepXML or tsv
            'digest_mass_range' : '{0} {1}'.format(
                self.params['translations']['_grouped_by_translated_key'][
                    'precursor_min_mass']['precursor_min_mass'],
                self.params['translations']['_grouped_by_translated_key'][
                    'precursor_max_mass']['precursor_max_mass']
            )
        }

        write_exclusion_list = [
            'precursor_min_mass',
            'precursor_max_mass',
            'precursor_min_charge',
            'precursor_max_charge',
            'label',
            '-Xmx',
            'header_translations',
            'validation_score_field'
        ]

        additional_15N_modifications = []
        if self.params['translations']['_grouped_by_translated_key']['label']['label'] == '15N':
            self.print_info(
                'Search with label=15N may still be errorprone. Evaluate with care!',
                caller='WARNING'
            )
            for aminoacid, N15_Diff in ursgal.ukb.DICT_15N_DIFF.items():
                existing = False
                for mod_dict in self.params['mods']['fix']:
                    if aminoacid == mod_dict['aa']:
                        mod_dict['mass'] += N15_Diff
                        mod_dict['name'] += '_15N_{0}'.format(aminoacid)
                        existing = True
                if existing == True:
                    continue
                else:
                    mod_key = 'add_{0}_{1}'.format(
                        aminoacid,
                        ursgal.chemical_composition_kb.aa_names[aminoacid]
                    )
                    self.params_to_write[mod_key] = N15_Diff

        for msfragger_param_name in self.params['translations']['_grouped_by_translated_key'].keys():
            for ursgal_param_name, param_value in self.params['translations']['_grouped_by_translated_key'][msfragger_param_name].items():
                if msfragger_param_name in write_exclusion_list:
                    continue
                elif msfragger_param_name == 'enzyme':
                    '''
                    search_enzyme_name = Trypsin
                    search_enzyme_cutafter = KR
                    search_enzyme_butnotafter = P
                    '''
                    aa_site, term, inhibitor = param_value.split(';')
                    self.params_to_write[
                        'search_enzyme_name'] = self.params['enzyme']
                    self.params_to_write['search_enzyme_cutafter'] = aa_site
                    self.params_to_write[
                        'search_enzyme_butnotafter'] = inhibitor
                elif msfragger_param_name == 'num_enzyme_termini':
                    # num_enzyme_termini = 2 # 2 for enzymatic, 1 for
                    # semi-enzymatic, 0 for nonspecific digestion

                    if self.params['translations']['_grouped_by_translated_key']['enzyme']['enzyme'] == 'nonspecific':
                        self.params_to_write[msfragger_param_name] = 0
                    else:
                        self.params_to_write[
                            msfragger_param_name] = param_value
                elif msfragger_param_name == 'clear_mz_range':
                    min_mz, max_mz = param_value
                    self.params_to_write[
                        msfragger_param_name] = '{0} {1}'.format(min_mz, max_mz)
                elif msfragger_param_name == 'modifications':
                    '''
                    #maximum of 7 mods - amino acid codes, * for any amino acid, [ and ] specifies protein termini, n and c specifies peptide termini
                    variable_mod_01 = 15.9949 M
                    variable_mod_02 = 42.0106 [*
                    #variable_mod_03 = 79.96633 STY
                    #variable_mod_03 = -17.0265 nQnC
                    #variable_mod_04 = -18.0106 nE
                    '''
                    # print(self.params['translations']['_grouped_by_translated_key'][msfragger_param_name])
                    # pprint.pprint(self.params[ 'mods' ])
                    # exit()
                    mass_to_mod_aa = ddict(list)
                    for mod_dict in self.params['mods']['opt']:
                        '''
                        {'_id': 0,
                          'aa': '*',
                          'composition': {'C': 2, 'H': 2, 'O': 1},
                          'id': '1',
                          'mass': 42.010565,
                          'name': 'Acetyl',
                          'org': '*,opt,Prot-N-term,Acetyl',
                          'pos': 'Prot-N-term',
                          'unimod': True},
                        '''
                        aa_to_append = mod_dict['aa']
                        pos_modifier = None
                        if mod_dict['pos'] == 'Prot-N-term':
                            pos_modifier = '['
                        elif mod_dict['pos'] == 'Prot-C-term':
                            pos_modifier = ']'
                        elif mod_dict['pos'] == 'N-term':
                            pos_modifier = 'n'
                        elif mod_dict['pos'] == 'C-term':
                            pos_modifier = 'c'
                        elif mod_dict['pos'] == 'any':
                            pass
                        else:
                            print(
                                '''
                            Unknown positional argument for given modification:
                            {0}
                            MSFragger cannot deal with this, please use one of the follwing:
                            any, Prot-N-term, Prot-C-term, N-term, C-term
                            '''.format(mod_dict['org'])
                            )
                            sys.exit(1)
                        if pos_modifier is not None:
                            aa_to_append = '{0}{1}'.format(
                                pos_modifier, aa_to_append)
                        mass_to_mod_aa[mod_dict['mass']].append(aa_to_append)
                    for pos, (mass, aa_list) in enumerate(mass_to_mod_aa.items()):
                        self.params_to_write['variable_mod_0{0}'.format(pos + 1)] = '{0} {1}'.format(
                            mass,
                            ''.join(aa_list)
                        )
                    for mod_dict in self.params['mods']['fix']:
                        '''
                        add_C_cysteine = 57.021464             # added to C - avg. 103.1429, mono. 103.00918
                        '''
                        if mod_dict['pos'] == 'Prot-N-term':
                            mod_key = 'add_Nterm_protein'
                        elif mod_dict['pos'] == 'Prot-C-term':
                            mod_key = 'add_Cterm_protein'
                        elif mod_dict['pos'] == 'N-term':
                            mod_key = 'add_Nterm_peptide'
                        elif mod_dict['pos'] == 'C-term':
                            mod_key = 'add_Cterm_peptide'
                        else:
                            mod_key = 'add_{0}_{1}'.format(
                                mod_dict['aa'],
                                ursgal.chemical_composition_kb.aa_names[
                                    mod_dict['aa']]
                            )
                        self.params_to_write[mod_key] = mod_dict['mass']

                elif msfragger_param_name == 'override_charge':
                    self.params_to_write[msfragger_param_name] = param_value
                    if param_value == 1:
                        self.params_to_write['precursor_charge'] = '{0} {1}'.format(
                            self.params['translations']['_grouped_by_translated_key'][
                                'precursor_min_charge']['precursor_min_charge'],
                            self.params['translations']['_grouped_by_translated_key'][
                                'precursor_max_charge']['precursor_max_charge']
                        )

                else:
                    self.params_to_write[msfragger_param_name] = param_value
        self.write_params_file()

        self.input_file = os.path.join(
            self.params['input_dir_path'],
            self.params['input_file']
        )
        if self.input_file.lower().endswith('.mzml') or \
                self.input_file.lower().endswith('.mzml.gz') or \
                self.input_file.lower().endswith('.mgf'):
            self.params['translations']['mzml_input_file'] = self.input_file
        # elif self.input_file.lower().endswith('.mgf'):
        #     self.params['translations']['mzml_input_file'] = \
        #         self.meta_unodes['ucontroller'].get_mzml_that_corresponds_to_mgf( self.input_file )
        #     self.print_info(
        #         'MSFragger can only read Proteowizard MGF input files,'
        #         'the corresponding mzML file {0} will be used instead.'.format(
        #             os.path.abspath(self.params['translations']['mzml_input_file'])
        #         ),
        #         caller = "INFO"
            # )
        else:
            raise Exception(
                'MSFragger input spectrum file must be in mzML or MGF format!')

        # pprint.pprint(self.params['translations'])
        # exit()
        self.params['command_list'] = [
            'java',
            '-Xmx{0}'.format(
                self.params['translations'][
                    '_grouped_by_translated_key']['-Xmx']['-xmx']
            ),
            '-jar',
            self.exe,
            self.param_file_name,
            self.params['translations']['mzml_input_file']
        ]

        self.params['translations']['output_file_incl_path'] = os.path.join(
            self.params['output_dir_path'],
            self.params['output_file']
        )
        return self.params

    def postflight(self):
        '''
        Reads MSFragger tsv output and write final csv output file.

        Adds:
            * Raw data location, since this can not be added later
            * Converts masses in Da to m/z (could be done in unify_csv)


        '''
        ms_fragger_header = [
            'ScanID',
            'Precursor neutral mass (Da)',
            'Retention time (minutes)',
            'Precursor charge',
            'Hit rank',
            'Peptide Sequence',
            'Upstream Amino Acid',
            'Downstream Amino Acid',
            'Protein',
            'Matched fragment ions',
            'Total possible number of matched theoretical fragment ions',
            # (including any variable modifications) (Da)
            'Neutral mass of peptide',
            'Mass difference',
            'Number of tryptic termini',
            'Number of missed cleavages',
            # '(starts with M, separated by |, formated as position,mass)
            'Variable modifications detected',
            'Hyperscore',
            'Next score',
            'Intercept of expectation model (expectation in log space)',
            'Slope of expectation model (expectation in log space)',
        ]

        translated_headers = []
        header_translations = self.UNODE_UPARAMS[
            'header_translations']['uvalue_style_translation']
        for original_header_key in ms_fragger_header:
            ursgal_header_key = header_translations[original_header_key]
            translated_headers.append(ursgal_header_key)

        translated_headers += [
            'Spectrum Title',
            'Raw data location',
            'Exp m/z',
            'Calc m/z',

        ]

        msfragger_output_tsv = os.path.join(
            self.params['input_dir_path'],
            self.params['file_root'] + '.tsv'
        )

        if os.path.exists(msfragger_output_tsv) is False:
            msfragger_output_tsv = os.path.join(
                self.params['input_dir_path'],
                self.params['file_root'][len(self.params['prefix'])+1:] + '.tsv'
            )    
            if os.path.exists(msfragger_output_tsv) is False:
                msfragger_output_tsv = os.path.join(
                    self.params['input_dir_path'],
                    '_'.join(self.params['file_root'].split('_')[1:]) + '.tsv'
                )
                if os.path.exists(msfragger_output_tsv) is False:
                    print('[ERROR]: MSFragger could not find the correct output tsv file')

        csv_out_fobject = open(self.params['translations'][
                               'output_file_incl_path'], 'w')
        csv_writer = csv.DictWriter(
            csv_out_fobject,
            fieldnames=translated_headers
        )
        csv_writer.writeheader()

        with open(msfragger_output_tsv) as temp_tsv:
            csv_reader = csv.DictReader(
                temp_tsv,
                fieldnames=translated_headers,
                delimiter='\t'
            )
            for line_dict in csv_reader:
                line_dict['Raw data location'] = os.path.abspath(
                    self.params['translations']['mzml_input_file']
                )

                ############################################
                # all fixing here has to go into unify csv! #
                ############################################

                # 'Precursor neutral mass (Da)' : '',
                # 'Neutral mass of peptide' : 'Calc m/z',# (including any variable modifications) (Da)
                line_dict['Exp m/z'] = ursgal.ucore.calculate_mz(
                    line_dict['MSFragger:Precursor neutral mass (Da)'],
                    line_dict['Charge']
                )
                line_dict['Calc m/z'] = ursgal.ucore.calculate_mz(
                    line_dict['MSFragger:Neutral mass of peptide'],
                    line_dict['Charge']
                )
                csv_writer.writerow(line_dict)

        csv_out_fobject.close()
        if msfragger_output_tsv.endswith('.tsv'):
            os.remove(msfragger_output_tsv)
        return



#!/usr/bin/env python
# encoding: utf-8
'''

'''

import os
import pyGCluster
import csv
import sys

def main(input_file=None, output_file=None, params=None):
    '''
    Arguments:
        input_file (str): input filename of csv which should be unified
        output_file (str): output filename of csv after unifying
        params (dict): params as passed by ursgal

    Note:
        Please do not forget to cite pyGCluster AND Ursgal when using this node.
        Thank you in advance!

    Please visit pyGCluster documentation for more information on this plotting
    function::

         * http://pygcluster.github.io/usage.html#clustered-data-visualization
         * color gradients
         * box styles


    Available parameters for heatmap::

        * heatmap_identifier_field_name defines the fieldname in the csv to
          appear directly right of the heatmap rows. Tipically the gene or
          protein name (Default: 'Protein')
        * heatmap_annotation_field_name defines the fieldname for an additional
          annotation in the csv to appear directly right of the object name in
          the heatmap rows. Tipically the gene or protein name
          (Default : 'map to uniprot')
        * heatmap_max_value defines the maximum value for the color scale
          (Default: 3)
        * heatmap_min_value defines the maximum value for the color scale
          (Default: -3)
        * heatmap_color_gradient defines the color gradient for the
          visualization (Default: 'Spectral')
        * heatmap_box_style defines the box style for the
          visualization (Default: 'classic')
        * heatmap_value_suffix is the suffix of the column name for columns
          holding a value (Default: '_mean')
        * heatmap_error_suffix is the suffix of the column name for columns
          holding the error to the value (Default: '_std')
        * heatmap_column_positions defines the order of the columns for plotting


    Note:

        Use of force = True is recommended to cover changes in the csv input
        file.
        Default value suffix of the column name is '_mean' and '_std' for the
        error estimate.
        Please refer to the documentation for further details on parameters.

    '''
    csv_reader = csv.DictReader(
        open(input_file, 'r')
    )
    # pprint.pprint(params)
    params['additional_labels'] = {}
    if params['heatmap_column_positions'] == {}:
        params['all_conditions']    = set()
        for fieldname in csv_reader.fieldnames:
            if fieldname.endswith(params['heatmap_value_suffix']):
                params['all_conditions'].add(
                    fieldname.replace(
                        params['heatmap_value_suffix'],
                        ''
                    )  # this tag could also go into params
                )
        params['all_conditions'] = sorted(list(params['all_conditions']))
    else:
        params['all_conditions'] = [
            params['heatmap_column_positions'][k] for k in sorted(params['heatmap_column_positions'].keys())
        ]
    plot_collector = {}
    identifiers    = []
    forbidden_character_list = [ '>', '<' ]
    for line_dict in csv_reader:
        line_name = line_dict[ params['heatmap_identifier_field_name'] ]
        for character in forbidden_character_list:
            line_name = line_name.replace( character, '__' )

        plot_collector[ line_name ] = {}

        for condition in params['all_conditions']:
            try:
                ratio =  float(
                    line_dict['{0}{1}'.format(condition,params['heatmap_value_suffix'])]
                )
                sd    = float(
                    line_dict['{0}{1}'.format(condition,params['heatmap_error_suffix'])]
                )
                plot_collector[ line_name ][condition] = (ratio, sd)
            except:
                continue
        identifiers.append(line_name)
        if params['heatmap_annotation_field_name'] in line_dict.keys():
            annotation = line_dict[params['heatmap_annotation_field_name']]
            for character in forbidden_character_list:
                annotation = annotation.replace( character, '__' )
            params['additional_labels'][ line_name ] = [ ' ', annotation ]

    cluster                      = pyGCluster.Cluster()
    folder                       = os.path.dirname(output_file)
    cluster['Working directory'] = folder
    cluster.draw_expression_map(
        data                        = plot_collector,
        identifiers                 = identifiers,
        conditions                  = params['all_conditions'],
        additional_labels           = params['additional_labels'],
        min_value_4_expression_map  = params['heatmap_min_value'],
        max_value_4_expression_map  = params['heatmap_max_value'],
        expression_map_filename     = output_file,
        box_style                   = params['heatmap_box_style'],
        color_gradient              = params['heatmap_color_gradient'],
    )

    return

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# encoding: utf-8
"""
"""
import pyqms
import pymzml

import os
import pickle


def generate_result_pickle(
    mzml_files,
    fixed_labels,
    molecules,
    evidence_files,
    min_charge,
    max_charge,
    label,
    ms_level,
    label_percentile,
    evidence_score_field=None,
    mz_score_percentile=0.4,
    trivial_names=None,
    pyqms_params=None,
    verbose=True
):
    """DOCSTRING."""
    if isinstance(mzml_files, str):
        mzml_files = [mzml_files]
    print('[ -ENGINE- ] Parse Evidences')

    fixed_labels, evidences, molecules = pyqms.adaptors.parse_evidence(
        fixed_labels=fixed_labels,
        evidence_files=evidence_files,
        molecules=molecules,
        evidence_score_field=evidence_score_field
    )

    params = {
        'molecules': molecules,
        'charges': [
            x for x in range(min_charge, max_charge + 1)
        ],
        'params': pyqms_params,
        'metabolic_labels': {
            label: label_percentile,
        },
        'trivial_names': trivial_names,
        'fixed_labels': fixed_labels,
        'verbose': verbose,
        'evidences': evidences
    }
    print('[ -ENGINE- ] Set up Isotopolugue Library')
    lib = pyqms.IsotopologueLibrary(**params)

    print('[ -ENGINE- ] Matching isotopologues to spectra ..')
    results = None
    for mzml_file in mzml_files:
        run = pymzml.run.Reader(
            mzml_file,
            obo_version='1.1.0',
            extraAccessions=[('MS:1000016' , ['value','unitName'])]
        )

        mzml_file_basename = os.path.basename(mzml_file)
        for n, spec in enumerate(run):
            if spec['id'] == 'TIC':
                break
            if n % 100 == 0:
                print(
                    '[ -ENGINE- ] File : {0:^40} : '
                    'Processing spectrum {1}'.format(
                        mzml_file_basename,
                        n,
                    ),
                    end='\r'
                )
            scan_time, unit = spec.scan_time

            if unit == 'second':
                scan_time /= 60
            elif unit != 'minute':
                print('''
                    [Warning] The retention time unit is not recognized or not specified.
                    [Warning] It is assumed to be minutes and continues with that.
                ''')

            if spec['ms level'] == ms_level:
                results = lib.match_all(
                    mz_i_list=spec.centroidedPeaks,
                    file_name=mzml_file_basename,
                    spec_id=spec['id'],
                    spec_rt=scan_time,
                    results=results
                )
        print()
    return results


def main(
    mzml_file=None,
    output_file=None,
    pickle_name=None,
    evidence_files=None,
    fixed_labels=None,
    molecules=None,
    rt_border_tolerance=1,
    label='14N',
    label_percentile=0.0,
    min_charge=1,
    max_charge=5,
    evidence_score_field='PEP',
    ms_level=1,
    trivial_names=None,
    pyqms_params=None,
    verbose=True
):
    """DOCSTRING."""

    rt_summary_file = os.path.join(
        output_file
    )

    results = generate_result_pickle(
        mzml_file,
        fixed_labels,
        molecules,
        evidence_files,
        min_charge,
        max_charge,
        label,
        ms_level,
        label_percentile,
        evidence_score_field,
        trivial_names=trivial_names,
        pyqms_params=pyqms_params,
        verbose=True
    )

    with open(pickle_name, 'wb') as f:
        pickle.dump(results, f)

    results.write_rt_info_file(
        output_file=rt_summary_file,
        rt_border_tolerance=float(rt_border_tolerance)
    )
    results.calc_amounts_from_rt_info_file(
        rt_info_file=rt_summary_file,
        rt_border_tolerance=float(rt_border_tolerance)
    )

    return rt_summary_file




#!/usr/bin/python

from __future__ import print_function
from __future__ import division
import os
import os.path
import sys
import csv
import math
import subprocess
from pprint import pprint
import ursgal


class msblender_09_2015( ursgal.UNode ):
    """
    MSblender UNode
    Documentation at http://www.marcottelab.org/index.php/MSblender
    """
    META_INFO = {
        'edit_version'       : 1.00,
        'name'               : 'MSblender',
        'version'            : '09.2015',
        'release_date'       : '2015-9-1',
        'engine_type' : {
            'meta_engine' : True,
        },
        'input_extensions'   : ['.csv'],
        'output_extensions'  : ['.csv'],
        'in_development'     : True,
        'create_own_folder'  : False,
        'include_in_git'     : False,
        'distributable'      : False,
        'utranslation_style' : 'msblender_style_1',
        'engine' : {
            'linux' : {
                '64bit' : {
                    'exe'            : 'msblender',
                    'url'            : '',
                    'zip_md5'        : None,
                    'additional_exe' : [],
                },
            },
            'darwin' : {
                '64bit' : {
                    'exe'            : 'msblender',
                    'url'            : '',
                    'zip_md5'        : None,
                    'additional_exe' : [],
                },
            },
        },
        'citation' : \
            'Kwon T, Choi H, Vogel C, Nesvizhskii AI, Marcotte EM. (2011) '\
            'MSblender: A Probabilistic Approach for Integrating Peptide '\
            'Identifications from Multiple Database Search Engines.',
    }

    def __init__( self, *args, **kwargs ):
        super(msblender_09_2015, self).__init__(*args, **kwargs)

        self.RT_lookup = {}

        return


    def preflight( self ):
        '''
        Formatting the command line via self.params

        Settings file is created in the output folder
        and added to self.created_tmp_files (can be deleted)

        Returns:

            self.params(dict)

        http://www.marcottelab.org/index.php/MSblender#Pre-processing
        '''

        # first, make sure that the msblender input generation script is
        # in the same path as the executable :
        exe_dir = os.path.dirname( self.exe )
        make_msblender_in_py_path = os.path.join(
            exe_dir, "make-msblender_in.py"
        )
        assert os.path.isfile( make_msblender_in_py_path ), '''
            The MSblender python script 'make-msblender_in.py'
            was not found in {}'''.format( exe_dir )
        self.params["make-msblender_in"] = make_msblender_in_py_path

        self.params['output_file_incl_path'] = os.path.join(
            self.params['output_dir_path'],
            self.params['output_file']
        )

        self.params["input_files"]         = []
        self.params["engine_names"]        = []
        self.params["engine_column_names"] = []
        self.params["evalue_column_names"] = []
        for input_file_dict in self.params["input_file_dicts"]:
            search_engine  = input_file_dict["last_search_engine"]
            score_colname  = input_file_dict["last_search_engine_colname"]
            input_file_path = os.path.join(
                input_file_dict['dir'],
                input_file_dict['file']
            )
            evalue_colname = self.meta_unodes[ search_engine ].DEFAULT_PARAMS['evalue_field']
            self.params["engine_names"].append( search_engine )
            self.params["engine_column_names"].append( score_colname )
            self.params["evalue_column_names"].append( evalue_colname )
            self.params["input_files"].append( input_file_path )


        print('Converting ident CSVs to MSblender "logE hit list" format...')
        msbl_hit_files = []
        for i, ident_csv in enumerate( self.params["input_files"] ):
            print( "Converting file {0} of {1}...".format(
                i+1, len(self.params["input_files"]))
            )
            f = self.convert_ursgal_csv_to_msblender_input( ident_csv )
            msbl_hit_files.append( f )

        print('Generating MSblender config file...')
        self.params["msblender_conf"] = \
            self.make_msblender_config_file( msbl_hit_files )


        command_list_to_generate_input = [
            'python2.7',
            '{make-msblender_in}'.format(**self.params),  # path to make-msblender_in.py
            '{msblender_conf}'.format(**self.params),     # path to msblender_conf file
        ]

        print( 'Generating MSblender input file via command line call:' )
        print( '$ ' + ' '.join( command_list_to_generate_input ) )
        print()
        proc = subprocess.Popen(
            command_list_to_generate_input,
            stdout=subprocess.PIPE,
        )
        for line in proc.stdout:
            print(line.strip().decode('utf'))


        # make-msblender_in.py should have produced an input file with this name:
        msblender_input_file_path = \
            self.params["msblender_conf"] + ".msblender_in"
        assert os.path.isfile( msblender_input_file_path ), '''
            MSblender input file could not be found at that path:
            {}'''.format( msblender_input_file_path)
        self.params["msblender_in"] = msblender_input_file_path

        self.params["command_list"] = [
            self.exe,                                # path to msblender executable
            '{msblender_in}'.format(**self.params),  # path to msblender_in file
        ]


    def postflight( self ):
        # check if msblender output file was sucessfully created or not
        self.params["msblender_out"] = \
            self.params["msblender_in"] + ".msblender_out"
        assert os.path.isfile( self.params["msblender_out"] ), "MSblender output file was not found."
        msg = "MSblender output file successfully created: {msblender_out}"
        print( msg.format( **self.params ) )

        print( "\nFormatting MSblender output file..." )
        output_file = self.convert_msblender_out( self.params["msblender_out"] )
        print( "MSblender Summary CSV created: {output_file_incl_path}".format(**self.params) )


    def make_msblender_config_file( self, msblender_input_files ):
        '''
        MSblender needs a config file of this format:
            MyriMatch:       test.myrimatch.mvh_hit_list_best
            X!Tandem:        test.tandem_k.logE_hit_list_best
            ...
        This function generates that file.
        '''
        config_file_path = self.params['output_file_incl_path'] + ".msblender_conf"

        with open( config_file_path, "w" ) as cfg_file:
            for i, msb_file in enumerate( msblender_input_files ):

                search_engine = self.params["engine_names"][ i ]

                line = "    ".join( [ search_engine, msb_file ] )
                cfg_file.write( line + "\n" )

        self.created_tmp_files.append( config_file_path )  # mark for deletion

        return config_file_path


    def convert_ursgal_csv_to_msblender_input( self, csv_file ):
        '''
        csv_file: search engine output file, converted to csv
            using ursgal

        generates a new csv in msblender "hit_list" format
        (see http://www.marcottelab.org/index.php/MSblender#Pre-processing )
        with the same name as input file, but file extension
        replaced with ".msblender_input"
        '''

        output_header = [
            "#Spectrum_id",
            "Charge",
            "PrecursorMz",
            "MassDiff",
            "Peptide",
            "Protein",
            "MissedCleavages",
            "Score(-log10[E-value])",
        ]

        msblender_decoy_tag = 'xf_'  # msblender interprets a hit as
                        # decoy if spectrum ID starts with this tag,
                        # so we have to mark our decoys with this tag!

        #basename   = os.path.basename( csv_file )
        #fname, ext = os.path.splitext( basename )

        converted_file = csv_file + ".msblender_input"

        with open( csv_file , "r") as in_f:
            rows = (line for line in in_f if not line.startswith("#"))
            reader = csv.DictReader( rows )
            with open( converted_file, "w" ) as out_f:
                writer = csv.DictWriter(
                    out_f, output_header, delimiter='\t'
                )

                writer.writeheader()
                for in_row in reader:

                    # only consider 'the best' PSM for each spectrum
                    if "rank" in in_row:
                        if int(in_row["rank"]) != 1:
                            continue
                    # same effect as 'select-best-PSM.py' from msblender

                    out_row = {}

                    sequence          = in_row["Sequence"]
                    mods              = in_row["Modifications"]
                    spec_title        = in_row["Spectrum Title"]
                    basename          = spec_title.split(".")[0]
                    spec_id_start     = int(spec_title.split(".")[ 1])
                    spec_id_stop      = int(spec_title.split(".")[ 2])
                    input_file_charge = int(spec_title.split(".")[-1])
                    charge            = in_row["Charge"]

                    out_row["Peptide"]      = "#".join( [sequence, mods] )
                    out_row["#Spectrum_id"] = spec_title
                    out_row["Charge"]       = charge

                    out_row["PrecursorMz"] = \
                        in_row["Exp m/z"]

                    out_row["MassDiff"] = \
                        abs(
                            float(in_row["Exp m/z"])
                            - float(in_row["Calc m/z"])
                            )

                    prot_ID = in_row["proteinacc_start_stop_pre_post_;"]
                    if in_row["Is decoy"] == "true" or "decoy" in prot_ID:
                        out_row["Protein"] = msblender_decoy_tag + prot_ID
                    else:
                        out_row["Protein"] = prot_ID

                    RandK = [ aa for aa in sequence[:-1] if aa in ["R","K"] ]
                    out_row["MissedCleavages"] = len( RandK )

                    for e in self.params["evalue_column_names"]:
                        if e in in_row:
                            evalue = float( in_row[ e ] )
                            # no log can be calculated for evalues that are supposedly
                            # zero so they are replaced with evalues of a very small value
                            # (converters by the msblender team use the same fix)
                            if evalue == 0.0:
                                evalue = 1e-15

                    out_row["Score(-log10[E-value])"] = -math.log10( evalue )


                    # noting down some PSM information in a lookup dict, we need
                    # this info later when the final output file is generated
                    for spec_id in set([spec_id_start, spec_id_stop]):
                        # msblender output doesn't have start+stop, so we note down both, just in case
                        PSM  = ( sequence, basename, spec_id, input_file_charge )
                        info = ( in_row["Retention Time (s)"], in_row["proteinacc_start_stop_pre_post_;"] )
                        self.RT_lookup[ PSM ] = info


                    writer.writerow( out_row )

        self.created_tmp_files.append( converted_file )  # mark for deletion

        return converted_file


    def convert_msblender_out( self, msblender_out ):
        '''
        Format the MSblender output csv to match our standards.
        It should look just like the combine_FDR output csv.
        Also adds the estimated FDR for each hit!
        '''
        msblender_converted_out = self.params["output_file_incl_path"]

        # mapping the msblender score column names to the ones that we want in our output csv,
        # i.e. { 'xtandem_score' : 'X\!Tandem:MSblender_score' }
        score_colname_map = {}
        for i, engine in enumerate( self.params["engine_names"] ):
            msbl_score_colname = engine + "_score"
            pretty_engine_name = self.params["engine_column_names"][ i ]
            new_score_colname  = "".join( [
                "MSblender:",
                pretty_engine_name,
                ":score"
            ] )  # i.e. "MSblender:MyriMatch_score"
            score_colname_map[ msbl_score_colname ] = new_score_colname

        out_header = ["Spectrum ID",
                      "Sequence",
                      "Modifications",
                      "MSblender:mvScore",
                      "Charge",
                      "Retention Time (s)",
                      "Engines",
                      "Is decoy",
                      "Estimated FDR",
        ]
        out_header += list( score_colname_map.values() )
        out_header.append( "proteinacc_start_stop_pre_post_;" )

        # these counters will be used for FDR calculation:
        PSM_count   = 0
        decoy_count = 0

        with open( msblender_out, "r" ) as msblout:
            reader = csv.DictReader( msblout, delimiter='\t' )
            with open( msblender_converted_out, "w" ) as out:
                writer = csv.DictWriter( out, out_header )
                writer.writeheader()

                # sorting the csv by score to allow FDR calculation while iterating
                # sorting in reverse since a bigger mvScore is better
                # (it's the "posterior probability of correct identification", see paper)
                sorted_reader = sorted(
                    reader,
                    key = lambda x: float(x["mvScore"]),
                    reverse=True,
                )

                for in_row in sorted_reader:

                    msblender_score = in_row["mvScore"]

                    if float(msblender_score) > self.DEFAULT_PARAMS['FDR_cutoff']:
                        continue

                    if in_row["Decoy"] == "D":
                        is_decoy = "true"
                    elif in_row["Decoy"] == "F":
                        is_decoy = "false"
                    else:
                        raise ValueError('''
                            "Decoy"-column info is missing from MSblender output row!
                            It should be either "D" or "R"!
                            ''')

                    #    desired output format:
                    # Spectrum ID   Sequence  Modifications    Charge  Retention Time (s)
                    # Engines  Is decoy   X\!Tandem:PEP   OMSSA:PEP   proteinacc_start_stop_pre_post_;

                    spec           = in_row["Spectrum"]
                    spec_tokens    = spec.split(".")
                    spec_id        = int( spec_tokens[-3] )
                    charge         = spec_tokens[ -2 ]
                    sequence, mods = spec_tokens[ -1 ].split("#")
                    basename       = spec_tokens[  0 ]

                    out_row = {}

                    out_row["MSblender:mvScore"] = msblender_score
                    out_row["Spectrum ID"]       = spec_id
                    out_row["Charge"]            = charge
                    out_row["Sequence"]          = sequence
                    out_row["Modifications"]     = mods
                    out_row["Is decoy"]          = is_decoy

                    PSM  = ( sequence, basename, spec_id, int(charge) )
                    RT, proteinacc = self.RT_lookup[ PSM ]

                    out_row["Retention Time (s)"]               = RT
                    out_row["proteinacc_start_stop_pre_post_;"] = proteinacc

                    engines_list = []  # list indicating which engines found the PSM
                    for u_score, m_score in score_colname_map.items():
                        out_row[ m_score ] = in_row[ u_score ]
                        if in_row[ u_score ] != "":
                            engines_list.append(
                                m_score.split(":")[1]
                                # MSblender:MyriMatch:score -> MyriMatch
                            )
                    out_row["Engines"] = ";".join( engines_list )

                    # estimating the FDR for each row:
                    PSM_count += 1
                    if is_decoy == 'true':
                        decoy_count += 1
                    # calculate and store the estimated FDR for each identification
                    out_row['Estimated FDR'] = self.calc_FDR ( PSM_count, decoy_count )

                    writer.writerow( out_row )

        self.created_tmp_files.append( msblender_converted_out )  # mark for deletion

        return msblender_converted_out


    def calc_FDR ( self, PSM_count, false_positives ):
        '''
        calculate false discovery rate according to FDR Method 2
        (Käll et al. 2008) as explained by Jones et al. 2009
        '''
        true_positives  = PSM_count - (2 * false_positives)
        if true_positives <= 0:  # prevent FDR above 1. Not sure if this is done?
            return 1.0
        FDR = false_positives / ( false_positives + true_positives)
        return FDR


    def add_estimated_FDR( self, csv_rowlist ):
        '''
        Adding the estimated FDR (based on # of decoys).
        based on Jones et al. 2009: Algorithm 2, step 3
        careful, input file must be sorted by average score (low->high) already!
        '''

        # In certain datasets, decoy hits are not
        # observed at any score threshold for identifications made by all
        # three search engines. To correct for the size of the result set, an
        # artificial decoy hit is added at the end of each data series, such
        # that no identification has a combined FDR/PEP Score = 0.
        artificial_decoy = [ {'Is decoy' : 'true'} ]

        PSM_count   = 0
        decoy_count = 0

        # Traverse identifications from lowest AFS to highest:
        for row in csv_rowlist + artificial_decoy:
            PSM_count += 1

            if row['Is decoy'] == 'true':
                decoy_count += 1

            # calculate and store the estimated FDR (FDRest) for each identification according to FDR Method 2.
            row['Estimated FDR']  = calc_FDR ( PSM_count, decoy_count )
            row['_decoy_percent'] = decoy_count / PSM_count


if __name__ == "__main__":

    m = msblender_09_2015()

    #m.params = {
        #"output_file"          : "./msblender_final_output.csv",
        #"folder"               : ".",
        #"basename"             : "120813OTc1_NQL-AU-0314-LFQ-LCM-SG-04_048_14N",
        #"input_files"    : [     # merged ident csv files:
            #"14N_msgfplus_v9979_merged.csv",
            #"14N_myrimatch_2_1_138_merged.csv",
            #"14N_omssa_2_1_9_merged.csv",
            #"14N_xtandem_sledgehammer_merged.csv",
            #],
        #"score_fields"   : {
            #"xtandem"    : "X\!Tandem:whatever",
            #"omssa"      : "OMSSA:bla",
            #"msgfplus"   : "MS-GF+:whatever",
            #"myrimatch"  : "MyriMatch:doesnmatter",
            #},
        #"FDR_cutoff" : 0.01,
        #}

    m.exe = "./msblender"

    m.preflight()
    m._execute()
    m.postflight()



#!/usr/bin/env python
import ursgal
import pprint
import xml.etree.ElementTree
import os
import platform
import sys
import subprocess
import csv

if sys.platform != 'win32':
    csv.field_size_limit(sys.maxsize)

class omssa_2_1_9( ursgal.UNode ):
    """
    omssa_2_1_9 UNode

    Parameter options at http://www.ncbi.nlm.nih.gov/IEB/ToolBox/CPP_DOC/asn_spec/omssa.asn.html

    OMSSA 2.1.9 parameters at http://proteomicsresource.washington.edu/protocols06/omssa.php

    Reference:
    Geer LY, Markey SP, Kowalak JA, Wagner L, Xu M, Maynard DM, Yang X, Shi W, Bryant SH (2004) Open Mass Spectrometry Search Algorithm.

    """
    META_INFO = {
        'edit_version'       : 1.00,
        'name'               : 'OMSSA',
        'version'            : '2.1.9',
        'release_date'       : None,
        'engine_type' : {
            'protein_database_search_engine' : True,
        },
        'input_extensions'   : [ '.mgf' ],
        'output_extensions'  : ['.csv'],
        'create_own_folder'  : True,
        'in_development'     : False,
        'include_in_git'     : False,
        'distributable'      : True,
        'utranslation_style' : 'omssa_style_1',
        ### Below are the download information ###
        'engine' : {
            'darwin' : {
                '64bit' : {
                    'exe'            : 'omssacl',
                    'url'            : 'ftp://ftp.ncbi.nih.gov/pub/lewisg/omssa/2.1.9/omssa-2.1.9.macos.tar.gz',
                    'zip_md5'        : '07854ab2c8c6776ab948791e560867ab',
                    'additional_exe' : ['makeblastdb'],
                },
            },
            'linux' : {
                '64bit' : {
                    'exe'            : 'omssacl',
                    'url'            : 'ftp://ftp.ncbi.nih.gov/pub/lewisg/omssa/2.1.9/omssa-2.1.9.linux.tar.gz',
                    'zip_md5'        : '287c612ab4fe4b8b96b57dd81e0acfc0',
                    'additional_exe' : ['makeblastdb'],
                },
            },
            'win32' : {
                '64bit' : {
                    'exe'            : 'omssacl.exe',
                    'url'            : 'ftp://ftp.ncbi.nih.gov/pub/lewisg/omssa/2.1.9/omssa-2.1.9.win32.exe',
                    'zip_md5'        : '29ed537ebccfa2345207f24183bd9149',
                    'additional_exe' : ['makeblastdb'],
                },
                # '32bit' : {
                #     'exe'            : 'omssacl.exe',
                #     'url'            : 'ftp://ftp.ncbi.nih.gov/pub/lewisg/omssa/2.1.9/omssa-2.1.9.win32.exe',
                #     'zip_md5'        : 'a05a5cdd45fd8abcfc75b1236f8a2390',
                #     'additional_exe' : ['makeblastdb'],
                # },
            },
        },
        'mods_to_unimod_correction' : {
            # this dict holds corrections of wrong OMSSA to unimod assignments
            'TMT 6-plex on K' : {
                'unimod_id'       : '737',
                'ommsa_unimod_id' : '738', # this is TMT duplex in unimod
                'unimod_name'     : 'TMT6plex',
            },
            'TMT 6-plex on n-term peptide' : {
                'unimod_id'       : '737',
                'ommsa_unimod_id' : '738', # this is TMT duplex in unimod
                'unimod_name'     : 'TMT6plex',
                'aa_targets'      : ['N-term'] # override 'X' in OMSSA mods xml
            },
            'TMT duplex on K' : {
                'unimod_id'       : '738',
                'ommsa_unimod_id' : '738', # this is TMT duplex in unimod
                'unimod_name'     : 'TMT2plex'
            },
            'TMT duplex on n-term peptide' : {
                'unimod_id'       : '738',
                'ommsa_unimod_id' : '738', # this is TMT duplex in unimod
                'unimod_name'     : 'TMT2plex',
                'aa_targets'      : ['N-term'] # override 'X' in OMSSA mods xml
            }
        },
        'citation' : \
            'Geer LY, Markey SP, Kowalak JA, Wagner L, Xu M, Maynard DM, '\
            'Yang X, Shi W, Bryant SH (2004) Open Mass Spectrometry Search '\
            'Algorithm.',
    }

    def __init__(self, *args, **kwargs):
        super(omssa_2_1_9, self).__init__(*args, **kwargs)
        self.omssa_mod_Mapper = None

    def _load_omssa_xml(self):
        ''' Parsing through omssa mods to map omssa mods on unimods '''
        self.omssa_mod_Mapper = {}
        def _create_empty_tmp():
            tmp = {
                'aa_targets' : [],
            }
            return tmp
        tmp = _create_empty_tmp()
        xml_path = os.path.dirname( self.exe )

        xml_names = ['mods.xml', 'usermods.xml']
        for xml_name in xml_names:
            omssa_xml = os.path.join(
                xml_path,
                xml_name
            )
            self.print_info(
                'Parsing omssa xml ({0})'.format(
                     omssa_xml
                ),
                caller='__ini__'
            )
            for event, element in xml.etree.ElementTree.iterparse( omssa_xml ):
                if element.tag.endswith('MSModSpec_residues_E'):
                    tmp['aa_targets'].append( element.text )

                elif element.tag.endswith('MSMod'):
                    tmp['omssa_id'] = element.text
                    # tmp['MSMod'] = element.text # OMSSA ID!
                elif element.tag.endswith('MSModSpec_psi-ms'):
                    tmp['unimod_name'] = element.text
                elif element.tag.endswith('MSModSpec_unimod'):
                    tmp['unimod_id'] = element.text
                    # tmp['MSModSpec_psi-ms'] = element.text # UNIMOD Name
                elif element.tag.endswith('MSModSpec_name'):
                    tmp['omssa_name'] = element.text
                    additional = []
                    if 'protein' in tmp['omssa_name']:
                        additional.append( 'Prot' )
                    if 'n-term' in tmp['omssa_name']:
                        additional.append('N-term')
                    elif 'c-term' in  tmp['omssa_name']:
                        additional.append('C-term')
                    if len(additional) > 0:
                        tmp['aa_targets'].append( '-'.join( additional ))

                elif element.tag.endswith('MSModSpec'):
                    lookup_field = 'unimod_id'
                    try:
                        l_value = tmp[ lookup_field ]
                    except:
                        self.print_info(
                            'Skipping entry {0} (no unimod! map)'.format( tmp ),
                            caller='WARNING!'
                        )
                        tmp['aa_targets'] = []
                        continue
                    if tmp['omssa_name'] in self.META_INFO['mods_to_unimod_correction'].keys():
                        l_value = self.META_INFO['mods_to_unimod_correction'][tmp['omssa_name']]['unimod_id']
                        # for TMT mods OMSSA writes an 'X' as amino acid target, this breaks later code...
                        if 'aa_targets' in self.META_INFO['mods_to_unimod_correction'][tmp['omssa_name']].keys():
                            tmp['aa_targets'] = self.META_INFO['mods_to_unimod_correction'][tmp['omssa_name']]['aa_targets']
                    if l_value not in \
                            self.omssa_mod_Mapper.keys():
                        self.omssa_mod_Mapper[ l_value ] = {}
                    self.omssa_mod_Mapper[ l_value ][ tmp[ 'omssa_id' ] ] = {
                        'aa_targets' : tmp[ 'aa_targets' ],
                        'omssa_name' : tmp[ 'omssa_name' ]
                    }

                    tmp = _create_empty_tmp()
        return

    def preflight( self ):
        '''
        Formatting the command line via self.params

        unimod Modifications are translated to OMSSA modifications

        Returns:
                self.params(dict)
        '''
        if self.omssa_mod_Mapper is None:
            self._load_omssa_xml()

        # building command_list !
        translations = self.params['translations']['_grouped_by_translated_key']
        blastdb_suffixes = [ '.phr', '.pin', '.psq' ]
        blastdb_present = True
        for blastdb_suffix in blastdb_suffixes:
            blast_file = self.params['database'] + blastdb_suffix
            if os.path.exists( blast_file ) is False:
                blastdb_present = False
                break

        if blastdb_present is False:
            self.print_info(
                'Executing makeblastdb...',
                caller = 'Info'
            )
            proc = subprocess.Popen(
                [
                    os.path.join(
                        os.path.dirname( self.exe ),
                        'makeblastdb'
                    ),
                    '-in', self.params['translations']['database'],
                    '-dbtype', 'prot',
                    '-input_type', 'fasta',
                ],
                stdout=subprocess.PIPE,
            )
            for line in proc.stdout:
                print(line.strip().decode('utf'))

        if self.params['translations']['label'] == '15N':
            translations['-tem']['precursor_mass_type'] = '2'
            translations['-tom']['frag_mass_type'] = '2'

        # Modifications
        # ------------------------

        for param_key in ['_fixed_mods', '_opt_mods']:
            mod_type = param_key[1:4]
            modifications = ''
            self.params[ param_key ] = ''
            for mod in self.params[ 'mods' ][ mod_type ]:
                unimod_id_does_not_exist = False
                aa_can_not_be_mapped = True
                # print(mod['id'])
                # print(self.omssa_mod_Mapper.keys())
                if mod[ 'id' ] not in self.omssa_mod_Mapper.keys():
                    unimod_id_does_not_exist = True
                else:
                    if mod['aa'] == '*':
                        search_target = [ mod[ 'pos' ], ]
                    else:
                        search_target = [ mod['aa'], ]
                    for omssa_id in self.omssa_mod_Mapper[ mod[ 'id' ] ].keys():
                        if search_target == self.omssa_mod_Mapper[ mod[ 'id' ] ][ omssa_id ]['aa_targets']:
                            modifications += '{0},'.format( omssa_id )
                            aa_can_not_be_mapped = False
                            omssa_name = self.omssa_mod_Mapper[ mod[ 'id' ] ][ omssa_id ]['omssa_name']
                            self.lookups[ omssa_name ] = {
                                'name'       : mod[ 'name' ],
                                'aa_targets' : self.omssa_mod_Mapper[ mod[ 'id' ] ][ omssa_id ]['aa_targets'],
                                'omssa_id'   : omssa_id,
                                'id'         : mod['id']
                            }
                # print(unimod_id_does_not_exist)
                # print(aa_can_not_be_mapped)
                if unimod_id_does_not_exist or aa_can_not_be_mapped:
                    self.print_info( '''
    The combination of modification name and aminoacid is not supported by
    OMSSA. Continuing without modification: {0}
                    '''.format(mod),
                    caller = 'WARNING'
                    )
                    continue

            self.params[ param_key ] = modifications.strip(',')

        # exit(1)
        # semi-enyzmatic cleavage --> translation into omssa enzyme number
        if self.params['translations']['semi_enzyme'] is True:
            if translations['-e']['enzyme'] == '0':
                translations['-e']['enzyme'] = '16'
            elif translations['-e']['enzyme'] == '3':
                translations['-e']['enzyme'] = '23'
            elif translations['-e']['enzyme'] == '13':
                translations['-e']['enzyme'] = '24'

        if self.params['translations']['frag_mass_tolerance_unit'] == 'ppm':
            translations['-to']['frag_mass_tolerance'] = \
                ursgal.ucore.convert_ppm_to_dalton(
                    self.params['translations']['frag_mass_tolerance'],
                    base_mz=self.params['translations']['base_mz']
                )

        print(
            '''
            [ WARNING ] precursor_mass_tolerance_plus and precursor_mass_tolerance_minus
            [ WARNING ] need to be combined for pyQms (use of symmetric tolerance window).
            [ WARNING ] The arithmetic mean is used.
            '''
        )
        self.params['_omssa_precursor_error'] = (
            float(self.params['translations']['precursor_mass_tolerance_plus']) +
            float(self.params['translations']['precursor_mass_tolerance_minus'])
        ) / 2.0

        self.params['_tmp_output_file_incl_path'] = os.path.join(
            self.params['output_dir_path'],
            self.params['output_file'] + '_tmp'
        )
        self.created_tmp_files.append( self.params['_tmp_output_file_incl_path'] )

        self.params['translations']['output_file_incl_path'] = os.path.join(
            self.params['output_dir_path'],
            self.params['output_file']
        )
        self.params['translations']['mgf_input_file'] = os.path.join(
            self.params['input_dir_path'],
            self.params['input_file']
        )
        translations['-fm']['mgf_input_file'] = self.params['translations']['mgf_input_file']

        assert os.path.exists( self.params['translations']['mgf_input_file']  ), '''
        OMSSA requires .mgf input (which should have been generated
        automatically ...)
        '''

        self.params['command_list'] = [
            self.exe,  # path 2 omssa executable
            '-w',
        ]

        for translated_key, translation_dict in translations.items():
            translated_value = str(list(translation_dict.values())[0])
            if translated_key == ('-oc', '-ox'):
                self.params['command_list'].extend( (translated_value, self.params['_tmp_output_file_incl_path']) )
                continue
            elif translated_key == '-te':
                self.params['command_list'].extend( (translated_key, str(self.params['_omssa_precursor_error'])) )
                continue
            elif translated_key in ['-teppm', '-ni']:
                if translated_value != '':
                    self.params['command_list'].append(translated_value)
                else:
                    continue
            elif translated_key == ('-mv', 'mf'):
                if self.params['_opt_mods'] != '':
                    self.params['command_list'].extend( ('-mv', self.params['_opt_mods']) )
                if self.params['_fixed_mods'] != '':
                    self.params['command_list'].extend( ('-mf', self.params['_fixed_mods']) )
                continue
            elif translated_key == ('-i', '-sct', '-sb1'):
                ion_translation_dict = {
                    'a' : '0',
                    'b' : '1',
                    'c' : '2',
                    'x' : '3',
                    'y' : '4',
                    'z' : '5',
                }
                ions_2_add = []
                for ion, ommsa_nr in ion_translation_dict.items():
                    if ion in translation_dict['score_ion_list']:
                        ions_2_add.append(ommsa_nr)
                self.params['command_list'].extend(('-i', ','.join(sorted(ions_2_add))))
                if 'b1' in translation_dict['score_ion_list']:
                    self.params['command_list'].extend(('-sb1', '0'))
                if 'c_terminal' in translation_dict['score_ion_list']:
                    self.params['command_list'].extend(('-sct', '0'))
                continue
            elif translated_key in [
                ('-tem','-tom'),
                'label',
                'semi_enzyme',
                'base_mz',
                'header_translations',
                'frag_mass_tolerance_unit',
                'output_file_incl_path',
            ]:
                continue
            elif len(translation_dict) == 1:
                self.params['command_list'].extend( (translated_key, translated_value) )
            else:
                print('The translatd key ', translated_key, ' maps on more than one ukey, but no special rules have been defined')
                print(translation_dict)
                sys.exit(1)

    def postflight( self ):
        '''
        Will correct the OMSSA headers and add the column retention time to the
        csv file
        '''
        cached_omssa_output = []
        result_file = open( self.params['_tmp_output_file_incl_path'], 'r')
        csv_dict_reader_object = csv.DictReader(
            row for row in result_file if not row.startswith('#')
        )
        headers = csv_dict_reader_object.fieldnames
        header_translations = self.UNODE_UPARAMS['header_translations']['uvalue_style_translation']
        translated_headers = []
        for header in headers:
            if header in [' E-value',' P-value']:
                continue
            translated_headers.append(
                header_translations.get(header, header)
            )
        translated_headers += [
            'Is decoy',
            'Retention Time (s)',
            header_translations.get(' E-value', ' E-value'),
            header_translations.get(' P-value', ' P-value'),
            'Raw data location',
        ]
        print('[ PARSING  ] Loading unformatted OMSSA results ...')
        for line_dict in csv_dict_reader_object:
            cached_omssa_output.append( line_dict )
        result_file.close()

        #
        result_file = open( self.params['translations']['output_file_incl_path'], 'w')
        csv_dict_writer_object = csv.DictWriter(
            result_file,
            fieldnames = translated_headers
        )
        csv_dict_writer_object.writeheader()
        #
        # self.parse_fasta()
        # already_seen_protein_scan_start_stop_combos = set()
        # database = self.params['translations']['database']
        for m in cached_omssa_output:
            tmp = {}
            for header in headers:
                translated_header = header_translations.get(
                    header,
                    header
                )
                tmp[ translated_header ] = m[ header ]
            tmp['Sequence'] = tmp['Sequence'].upper()

            translated_mods = []
            if tmp['Modifications'] != '':
                splitted_Modifications = tmp['Modifications'].split(',')
                for mod in splitted_Modifications:
                    omssa_name, position = mod.split(':')
                    omssa_name  = omssa_name.strip()
                    position    = position.strip()
                    unimod_name = self.lookups[ omssa_name ]['name']
                    if position.strip() == '1':
                        # print( self.lookups[ omssa_name ] )
                        for target in self.lookups[ omssa_name ]['aa_targets']:
                            if 'N-TERM' in target.upper():
                                position = '0'
                    translated_mods.append(
                        '{0}:{1}'.format(
                            unimod_name,
                            position
                        )
                    )

            tmp['Modifications'] = ';'.join( translated_mods )
            tmp['Raw data location'] = self.params['translations']['mgf_input_file']

            csv_dict_writer_object.writerow( tmp )
        return





#!/usr/bin/env python3
# encoding: utf-8

import ursgal
import os
import sys


def main():
    '''
    Simple crosslink search using Kojak and an example file from Barth et al. 
    2014, please note that this is only testing if Kojak works.
    The sample represents no crosslink data in particular, but Kojak is used to
    map possible disulfide bonds.

    Parameters have not been optimized yet, please use this script as a template
    to use Kojak. Please note the different approach for executing Percolator.

    Note:

        Please note that Kojak has to installed manually at the resources folder
        under engine-folder (kojak_1_5_3). 
        Additinally Percolator (2.08) has to be symlinked or copied to
        engine-folder 'kojak_percolator_2_08'. 

    Usage:
        ./simple_srosslink_search.py <path_2_chlamydomonas_reinhardtii_database>

    Note:

        The peptide -.GHYLNATAGTC[-34.00]EEMMK.- from Rubisco large subunit is 
        detected with a predicted disulfide bond site at this position.
        This site is reported to have a disulfide bond in the Uniprot database.
        The modification (C +32, C -34) was set according to  'Tsai PL, Chen SF,
        Huang SY (2013) Mass spectrometry-based strategies for protein disulfide 
        bond identification. Rev Anal Chem 32: 257–268'
        Please use the reference C. reinhardtii (TaxId 3055 ) proteome from
        Unipot.

    '''

    params = {
        'database': sys.argv[1],
        'ftp_url': 'ftp.peptideatlas.org',
        'ftp_login': 'PASS00269',
        'ftp_password': 'FI4645a',
        'ftp_include_ext': [
            'JB_FASP_pH8_2-3_28122012.mzML',
        ],
        'ftp_output_folder': os.path.join(
            os.pardir,
            'example_data',
            'simple_crosslink_search'
        ),
        'cross_link_definition': ['C C -2 test_if_kojak_runs'],
        'mono_link_definition': ['C 32', 'C -34'],
        'modifications': [
            'M,opt,any,Oxidation',        # Met oxidation
        ],
        'precursor_min_mass': 500,
        'precursor_max_mass': 8000,
        'precursor_mass_tolerance_plus': 15,
        'precursor_mass_tolerance_minus': 15,
        'max_accounted_observed_peaks': 0,  # i.e. all
        'max_num_mods': 2
    }

    if os.path.exists(params['ftp_output_folder']) is False:
        os.mkdir(params['ftp_output_folder'])

    uc = ursgal.UController(
        profile='LTQ XL low res',
        params=params
    )
    mzML_file = os.path.join(
        params['ftp_output_folder'],
        params['ftp_include_ext'][0]
    )
    if os.path.exists(mzML_file) is False:
        uc.fetch_file(
            engine='get_ftp_files_1_0_0'
        )

    td_database_name = params['database'].replace(
        '.fasta', '_target_decoy.fasta')
    if os.path.exists(td_database_name) is False:
        uc.execute_misc_engine(
            input_files=[params['database']],
            engine='generate_target_decoy_1_0_0',
            output_file_name=td_database_name,
        )
    uc.params['database'] = td_database_name

    engine = 'kojak_1_5_3'

    mgf_file = uc.convert(
        input_file=mzML_file,
        engine='mzml2mgf',
    )
    raw_result = uc.search_mgf(
        input_file=mgf_file,
        engine=engine,
        force=False,
    )
    for extension in uc.unodes[engine]['META_INFO']['all_extensions']:
        if 'perc' not in extension:
            continue
        file_to_validate = raw_result.replace(
            uc.unodes[engine]['META_INFO']['output_extension'],
            extension
        )
        try:
            uc.validate(
                file_to_validate,
                'kojak_percolator_2_08'
            )
        except:
            pass

    return


if __name__ == '__main__':
    main()



#!/usr/bin/env python
import ursgal
import os
import subprocess
import csv
import re
import sys


class moda_v1_51(ursgal.UNode):
    """
    MODa UNode
    Check http://prix.hanyang.ac.kr/download/moda.jsp for download, new versions and contact information

    Reference:
    Na S, Bandeira N, Paek E. (2012) Fast multi-blind modification search through tandem mass spectrometry.
    """
    META_INFO = {
        'edit_version': 1.00,
        'name': 'MODa',
        'version': 'v1.51',
        'release_date': '2012-4-1',
        'engine_type': {
            'protein_database_search_engine': True,
        },
        'input_extensions': ['.mgf', '.pkl', '.dta', '.mzXML'],
        'output_extensions': ['.csv'],
        'create_own_folder': True,
        'in_development': False,
        'include_in_git': False,
        'distributable': False,
        'utranslation_style': 'moda_style_1',
        'engine': {
            'platform_independent': {
                'arc_independent': {
                    'exe': 'moda_v1.51.jar',
                    'url': '',
                    'zip_md5': '',
                    'additional_exe': [],
                },
            },
        },
        'citation':
        'Na S, Bandeira N, Paek E. (2012) Fast multi-blind modification '
            'search through tandem mass spectrometry.',
    }

    def __init__(self, *args, **kwargs):
        super(moda_v1_51, self).__init__(*args, **kwargs)
        pass

    def preflight(self):
        '''
        Formatting the command line via self.params

        Returns:
                dict: self.params
        '''

        translations = self.params['translations'][
            '_grouped_by_translated_key']

        self.params['translations']['params_input_file'] = os.path.join(
            self.params['output_dir_path'],
            self.params['output_file'] + '_params.txt'
        )
        self.created_tmp_files.append(
            self.params['translations']['params_input_file'])

        self.params['translations']['output_file_incl_path'] = os.path.join(
            self.params['output_dir_path'],
            self.params['output_file']
        )
        self.created_tmp_files.append(
            self.params['translations'][
                'output_file_incl_path'].replace('.csv', '.tsv')
        )
        translations['-o']['output_file_incl_path'] = \
            self.params['translations']['output_file_incl_path']

        self.params['command_list'] = [
            'java',
            '-jar',
            self.exe,
            '-i', self.params['translations']['params_input_file'],
            '-o', self.params['translations'][
                'output_file_incl_path'].replace('.csv', '.tsv'),
        ]

        self.params['translations']['mgf_input_file'] = os.path.join(
            self.params['input_dir_path'],
            self.params['input_file']
        )
        translations['Spectra']['mgf_input_file'] = \
            self.params['translations']['mgf_input_file']

        params_input_file = open(
            self.params['translations']['params_input_file'],
            'w',
            encoding='UTF-8'
        )

        fixed_mods = []
        if self.params['translations']['label'] == '15N':
            for aminoacid, N15_Diff in ursgal.ukb.DICT_15N_DIFF.items():
                existing = False
                for mod in self.params['mods']['fix']:
                    if aminoacid == mod['aa']:
                        mod['mass'] += N15_Diff
                        mod['name'] += '_15N_{0}'.format(aminoacid)
                        existing = True
                if existing == True:
                    continue
                else:
                    fixed_mods.append('{0}, {1}'.format(aminoacid, N15_Diff))

        for mod in self.params['mods']['fix']:
            fixed_mods.append('{0}, {1}'.format(mod['aa'], mod['mass']))

        print(
            '''
            [ WARNING ] precursor_mass_tolerance_plus and precursor_mass_tolerance_minus
            [ WARNING ] need to be combined for MODa (use of symmetric tolerance window).
            [ WARNING ] The arithmetic mean is used.
            '''
        )
        if translations['PPMTolerance']['precursor_mass_tolerance_unit'] == 'da':
            translations['PeptTolerance'] = \
                (translations['PPMTolerance']['precursor_mass_tolerance_minus'] +
                 translations['PPMTolerance']['precursor_mass_tolerance_plus']) / 2
            del translations['PPMTolerance']

        elif translations['PPMTolerance']['precursor_mass_tolerance_unit'] == 'mmu':
            translations['PeptTolerance'] = \
                10e-3 *\
                (translations['PPMTolerance']['precursor_mass_tolerance_minus'] +
                 translations['PPMTolerance']['precursor_mass_tolerance_plus']) / 2
            del translations['PPMTolerance']

        else:
            translations['PPMTolerance'] = \
                (translations['PPMTolerance']['precursor_mass_tolerance_minus'] +
                 translations['PPMTolerance']['precursor_mass_tolerance_plus']) / 2

        if translations['FragTolerance']['frag_mass_tolerance_unit'] == 'ppm':
            translations['FragTolerance'] = \
                ursgal.ucore.convert_ppm_to_dalton(
                    translations['FragTolerance']['frag_mass_tolerance'],
                    base_mz=self.params['translations']['base_mz']
            )
        elif translations['FragTolerance']['frag_mass_tolerance_unit'] == 'mmu':
            translations['FragTolerance'] = \
                translations['FragTolerance']['frag_mass_tolerance'] * 10e-3
        else:
            translations['FragTolerance'] = \
                translations['FragTolerance']['frag_mass_tolerance']

        for translated_key, translation_dict in translations.items():
            if translated_key == '-Xmx':
                self.params['command_list'].insert(1, '{0}{1}'.format(
                    translated_key,
                    list(translation_dict.values())[0]
                ))
            elif translated_key in ['-o', '-i', 'base_mz', 'label', ]:
                continue
            elif translated_key == 'ADD':
                for aa_mod in fixed_mods:
                    print('{0}={1}'.format(
                        translated_key,
                        aa_mod,
                    ),
                        file=params_input_file
                    )
            elif translated_key in ['FragTolerance', 'PPMTolerance', 'PeptTolerance']:
                print('{0}={1}'.format(
                    translated_key,
                    translations[translated_key],
                ),
                    file=params_input_file
                )
            elif len(translation_dict) == 1:
                print('{0}={1}'.format(
                    translated_key,
                    str(list(translation_dict.values())[0]),
                ),
                    file=params_input_file
                )
            else:
                print('The translatd key ', translated_key,
                      ' maps on more than one ukey, but no special rules have been defined')
                print(translation_dict)
                sys.exit(1)

        params_input_file.close()

        return self.params

    def postflight(self):
        '''
        Rewrite ModA output .tsv into .csv so that it can be unified
        '''
        output_headers = [
            'Spectrum Title',
            'Retention Time (s)',
            'Spectrum ID',
            'Sequence',
            'Modifications',
            'Protein ID',
            'Exp m/z',
            'Calc m/z',
            'Error (exp-calc)',
            'ModA:Score',
            'ModA:probability',
            'Spectrum ID',
            'Charge',
            'Rank',
            'Raw data location',
        ]
        org_moda_out = self.params['translations'][
            'output_file_incl_path'].replace('.csv', '.tsv')
        out_line_dicts = []
        with open(org_moda_out, 'r') as org_out:
            out_reader = csv.reader(org_out, delimiter='\t')
            for row in out_reader:
                if len(row) <= 1:
                    continue
                elif '>>' in row[0]:
                    out_dict = {}
                    rank = 0
                    out_dict['Raw data location'] = row[0].strip('>>')
                    out_dict['Exp m/z'] = row[2]
                    out_dict['Charge'] = row[3]
                    out_dict['Spectrum ID'] = row[4]
                else:
                    rank += 1
                    out_dict['Calc m/z'] = row[0]
                    out_dict['Error (exp-calc)'] = row[1]
                    out_dict['ModA:Score'] = row[2]
                    out_dict['ModA:probability'] = row[3]
                    org_sequence = row[4].split('.')[1]
                    out_dict['Protein ID'] = row[5]
                    out_dict['Rank'] = rank

                    tmp_sequence = ''
                    tmp_mods = []
                    regex_pattern = '([+-]{1}[0-9]+)'
                    peptide_unimod = ursgal.ucore.reformat_peptide(
                        regex_pattern,
                        None,
                        org_sequence
                    )
                    if '#' in peptide_unimod:
                        sequence, mods = peptide_unimod.split('#')
                    else:
                        sequence = peptide_unimod
                        mods = ''
                    out_dict['Sequence'] = sequence
                    out_dict['Modifications'] = mods
                    out_line_dicts.append(out_dict.copy())

                    # prev_match = 0
                    # for match in mod_pattern.finditer(org_sequence):
                    #     mod = match.group('mod')
                    #     pos = int(match.start())
                    #     tmp_mods.append('{0}:{1}'.format(mod, pos))
                    #     tmp_sequence += org_sequence[prev_match:pos]
                    #     prev_match = pos + len(mod)
                    # sequence = tmp_sequence + \
                    #     org_sequence[prev_match:len(org_sequence)]

                    # mods = ';'.join(tmp_mods)
                    # if tmp_mods == []:
                    #     sequence = org_sequence

        output_file = self.params['translations']['output_file_incl_path']
        with open(output_file, 'w') as out_file:
            csv_writer = csv.DictWriter(out_file, fieldnames=output_headers)
            csv_writer.writeheader()
            for line_dict in out_line_dicts:
                csv_writer.writerow(line_dict)

        return output_file



#!/usr/bin/env python
import ursgal
import math
import csv
import bisect
import os
from collections import defaultdict as ddict
from collections import OrderedDict
import sys
import pprint


class kojak_percolator_2_08( ursgal.UNode ):
    """
    Kojak adjusted Percolator 2_08 UNode

    Kojak provides preformatted Percolator input, this is used direclty as the
    input file for Percolator. In contrast to the original Percolator node, the
    input files are not reformatted or used to write a new input file.

    Note:

        Percolator (2.08) has to be symlinked or copied to engine-folder
        'kojak_percolator_2_08' in order to make this node work.

    Reference:
    Käll L, Canterbury JD, Weston J, Noble WS, MacCoss MJ. (2007) Semi-supervised learning for peptide identification from shotgun proteomics datasets.
    """
    META_INFO = {
        'edit_version'       : 1.00,
        'name'               : 'Kojak Percolator',
        'version'            : '2.08',
        'release_date'       : '2007-11-1',
        'engine_type' : {
            'validation_engine' : True,
        },
        'input_extensions'   : ['.txt'],
        'output_extensions'  : ['.tsv'],
        'output_suffix'      : 'percolator_2_08_validated',
        'create_own_folder'  : False,
        'distributable'  : False,
        'in_development'     : False,
        'include_in_git'     : None,
        'utranslation_style' : 'kojak_percolator_style_1',
        'engine' : {
            'darwin' : {
                '64bit' : {
                    'exe'            : 'percolator_2_08',
                    'url'            : '',
                    'zip_md5'        : '',
                    'additional_exe' : [],
                },
            },
            'linux' : {
                '64bit' : {
                    'exe'            : 'percolator',
                    'url'            : '',
                    'zip_md5'        : '',
                    'additional_exe' : [],
                },
            },
            'win32' : {
                '64bit' : {
                    'exe'            : 'percolator.exe',
                    'url'            : '',
                    'zip_md5'        : '',
                    'additional_exe' : [],
                },
                '32bit' : {
                    'exe'            : 'percolator.exe',
                    'url'            : '',
                    'zip_md5'        : '',
                    'additional_exe' : [],
                },
            },
        },
        'citation' : \
            'K・・ｽ､ll L, Canterbury JD, Weston J, Noble WS, MacCoss MJ. (2007) '\
            'Semi-supervised learning for peptide identification from shotgun '\
            'proteomics datasets.',
    }

    def __init__(self, *args, **kwargs):
        super(kojak_percolator_2_08, self).__init__(*args, **kwargs)
        pass

    def preflight( self ):
        '''
        Formatting the command line to via self.params
        '''

        self.params['translations']['input_file'] = os.path.join(
            self.params['input_dir_path'],
            self.params['input_file']
        )

        self.params['translations']['output_file_incl_path'] = os.path.join(
            self.params['output_dir_path'],
            self.params['output_file']
        )

        self.params['command_list'] = [
            # percolator -X pout.xml pin.tab >| yeast-01.psms
            self.exe,
            '--only-psms',
            '{input_file}'.format(**self.params['translations']),
            '--results-psms',
            '{output_file_incl_path}'.format(**self.params['translations']),
        ]


    def postflight( self ):
        '''
        Convert the percolator output .tsv into the .csv format with headers as in the
        unified csv format.
        '''
        new_output_file = self.params['translations']['output_file_incl_path'].replace('.perc.','_').replace('.tsv','.csv')
        percolator_output_dict_reader = csv.DictReader(
            open(
                self.params['translations']['output_file_incl_path'],
                'r'
            ),
            delimiter='\t'
        )
        translated_fieldnames = []
        for old_fieldname in percolator_output_dict_reader.fieldnames:
            new_fieldname = self.UNODE_UPARAMS['header_translations']['uvalue_style_translation'][old_fieldname]
            translated_fieldnames.append( new_fieldname )

        percolator_output_writer = csv.DictWriter(
            open(
                new_output_file,
                'w'
            ),
            fieldnames = translated_fieldnames
        )
        percolator_output_writer.writeheader()

        for pos, line_dict in enumerate(percolator_output_dict_reader):
            dict_2_write = {}
            proteinIds   = []
            for key, value in line_dict.items():
                if key in [ None, 'proteinIds']:
                    if key is None:
                        proteinIds += value
                    else:
                        proteinIds.append(value)
                else:
                    new_key = self.UNODE_UPARAMS['header_translations']['uvalue_style_translation'][key]

                    dict_2_write[new_key] = value
            dict_2_write['Protein ID'] = '__'.join(sorted(proteinIds))
            percolator_output_writer.writerow(dict_2_write)
        self.created_tmp_files += [
           self.params['translations']['output_file_incl_path']
        ]

        return



#!/usr/bin/env python
import ursgal
import importlib
import os
import sys


class mzml2mgf_2_0_0(ursgal.UNode):
    """
    mzml2mgf_2_0_0 UNode

    Version two works only with pymzML version 2.0.0 or higher!

    Converts .mzML files into .mgf files
    """
    META_INFO = {
        'edit_version'       : 1.00,
        'name'               : 'mzml2mgf',
        'version'            : '2.0.0',
        'release_date'       : '2018-03-26',
        'engine_type' : {
            'converter' : True,
        },
        'input_extensions'   : ['.mzML', '.mzML.gz', '.idx.gz'],
        'output_extensions'  : ['.mgf'],
        'output_suffix'      : None,
        'in_development'     : False,
        'include_in_git'     : True,
        'distributable'      : True,
        'utranslation_style' : 'mzml2mgf_style_1',
        'engine' : {
            'platform_independent' : {
                'arc_independent' : {
                    'exe' : 'mzml2mgf_2_0_0.py',
                },
            },
        },
        'citation' :
            'Kremer, L. P. M., Leufken, J., Oyunchimeg, P., Schulze, S. & '
            'Fufezan, C. (2016) Ursgal, Universal Python Module Combining '
            'Common Bottom-Up Proteomics Tools for Large-Scale Analysis. J. '
            'Proteome res. 15, 788-794.',
    }

    def __init__(self, *args, **kwargs):
        super(mzml2mgf_2_0_0, self).__init__(*args, **kwargs)

    def _execute(self):
        print('[ -ENGINE- ] Executing conversion ..')
        # self.time_point(tag = 'execution')
        mzml2mgf_main = self.import_engine_as_python_function()

        tmp = mzml2mgf_main(
            mzml=os.path.join(
                self.io['input']['finfo']['dir'],
                self.io['input']['finfo']['file']
            ),
            mgf=os.path.join(
                self.io['output']['finfo']['dir'],
                self.io['output']['finfo']['file']
            ),
            i_decimals=self.params['translations']['num_i_decimals'],
            mz_decimals=self.params['translations']['num_mz_decimals'],
            machine_offset_in_ppm=self.params[
                'translations']['machine_offset_in_ppm'],
            scan_exclusion_list=self.params[
                'translations']['scan_exclusion_list'],
            scan_inclusion_list=self.params[
                'translations']['scan_inclusion_list'],
            prefix=self.params.get('prefix', None),
            scan_skip_modulo_step=self.params[
                'translations']['scan_skip_modulo_step'],
            ms_level=self.params['translations']['ms_level'],
            precursor_min_charge=self.params[
                'translations']['precursor_min_charge'],
            precursor_max_charge=self.params[
                'translations']['precursor_max_charge'],
            ion_mode=self.params['translations']['ion_mode'],
            spec_id_attribute=self.params['translations']['pymzml_spec_id_attribute'],
        )
        return tmp



#!/usr/bin/env python3
# encoding: utf-8

import ursgal
import os


R = ursgal.UController()

TESTS = [
    {
        'input' : {
            'input_files' : [
                os.path.join( os.sep, 'Volumes', '_', '_', '_', '1') + os.sep,
                os.path.join( os.sep, 'Volumes', '_', '_', '_', '1') + os.sep
            ],
        },
        'output' : os.path.join( os.path.abspath( os.sep ), 'Volumes', '_', '_', '_', '1') + os.sep
    },
    {
        'input' : {
            'input_files' : [
                os.path.join( os.sep, 'Volumes', '_', '_', '_') + os.sep,
                os.path.join( os.sep, 'Volumes', '_', '_', '_', '1') + os.sep
            ],
        },
        'output' : os.path.join( os.path.abspath( os.sep ), 'Volumes', '_', '_', '_') + os.sep,
    },
    {
        'input' : {
            'input_files' : [
                os.path.join( os.sep, 'Volumes') + os.sep,
                os.path.join( os.sep, 'Volumes', '_', '_', '_', '1') + os.sep,
            ],
        },
        'output' : os.path.join( os.path.abspath( os.sep ), 'Volumes') + os.sep
    },

]


def determine_common_top_level_folder_test():
    for test_id, test_dict in enumerate(TESTS):
        yield determine_common_top_level_folder, test_dict


def determine_common_top_level_folder( test_dict ):
    out_put = R.determine_common_top_level_folder(
        **test_dict['input']
    )
    print( out_put , test_dict)
    assert out_put == test_dict['output'], '''
    determine_common_top_level_folder failed with
        test:
        {0}
        output
        {1}'''.format(
        test_dict,
        out_put
    )


if __name__ == '__main__':
    for test_id, test_dict in enumerate(TESTS):
        determine_common_top_level_folder( test_dict )



#!/usr/bin/env python
import ursgal
import os

class crux_2_1( ursgal.UNode ):
    """
    crux_2_1 UNode

    Not implemented yet
    """

    META_INFO = {
        'edit_version'      : 1.00,
        'name'              : 'Crux',
        'version'           : '2.1',
        'release_date'      : None,
        'engine_type' : {
            'protein_database_search_engine' : True,
        },
        'input_extensions'  : [],
        'output_extensions' : [],
        'in_development'    : True,
        'include_in_git'    : None,
        'distributable'      : False,
        'utranslation_style' : 'crux_style_1',
        'citation' : \
            '',
    }


    def __init__(self, *args, **kwargs):
        super(crux_2_1, self).__init__(*args, **kwargs)

        pass

    def print_param_line(self,key, value, io):
        '''
        '''
        print(
            '{0}={1}'.format(
                key,
                value),
            file=io
        )
        return


    def write_param_file(self):
        self.params['param_file'] = self.params['output_file_basename_incl_path'] + '_crux_params.txt'
        cam_mod = ursgal.ukb.CAM_MOD
        io = open(self.params['param_file'],'w')

        cam = False
        for mod in self.params[ 'mods' ][ 'fix' ]:
            if self.params['label'] == '15N' and mod[ 'aa' ] == 'C' and mod[ 'name' ] == 'Carbamidomethyl':
                cam = True
                continue
            print_param_line( mod[ 'aa' ], mod[ 'mass' ])

        potential_modifications = ''
        n_term_mods = ''
        for mod in self.params[ 'mods' ][ 'opt' ]:
            if mod[ 'pos' ] != 'any':
                continue
            potential_modifications += '{0}:{1}:10,'.format(mod[ 'mass' ], mod[ 'aa' ] )
        if potential_modifications != '':
            print_param_line( 'mod=', potential_modifications )

        for aminoacid, modification in ursgal.ukb.DICT_15N_DIFF.items():
            if aminoacid == 'C':
                if cam == True:
                    if self.params['label'] == '15N':
                        modification += cam_mod
                    else:
                        modification = cam_mod
                else:
                    modification = 0
                print_param_line(
                    aminoacid,
                    modification,
                    io
                )
            elif self.params['label'] == '15N':
                print_param_line(
                    aminoacid,
                    modification,
                    io
                )
        self.params_list =[
            ('spectrum-min-mz','150.000000'),
            ('min-peaks','15'),
            ('precursor-window','5'),
            ('precursor-window-type','ppm'),

        ]
        # if self.params['machine'] == 'QExactive+':
        #     self.params_list.append(('precursor-window','5'))
        #     # self.params['product_ion_tolerance'] = '0.02'

        # elif self.params['machine'] == 'LTQ XL':
        #     # self.params['ions_to_search'] = '1,4'
        #     # self.params['product_ion_tolerance'] = '0.5'
        #     pass


        for key, value in self.params_list:
            print_param_line(key, value, io)

        io.close()
        # return


    def preflight( self ):
        '''
        CRUX param options:
        http://cruxtoolkit.sourceforge.net/default.params


        CRUX tide-index options:
        http://cruxtoolkit.sourceforge.net/tide-index.html

        CRUX tide-seach options:
        http://cruxtoolkit.sourceforge.net/tide-search.html

        '''

        self.params['output_file'] = self.params['output_file_basename_incl_path']+'{0}.csv'.format(self.params['ident_csv_suffix'])

        self.params['fileroot'] = os.path.basename( self.params['output_file_basename_incl_path' ] )
        self.write_param_file()
        pprint.pprint(self.params)
        exit()
        # building command_list !
        #
        print('  Building database index for tide ...')
        self.params['command_list'] = [
            # creating the index
            '{executable_path}'.format(**self.params),
            'tide-index',
            '{database}'.format( **self.params ),
            '--decoy-format', #none|shuffle|peptide-reverse|protein-reverse
            'none',
            '{database}_crux_dbindex'.format(**self.params),
            '--overwrite', 'T',  #Replace existing files if true (T) or fail when trying to overwrite a file if false (F)
            '--parameter-file', #A file containing command-line or additional parameters
            '{param_file}'.format(**self.params),
            '--output-dir', #The name of the directory where output files will be created
            '{output_folder}'.format(**self.params),
            '--fileroot',   #The fileroot string will be added as a prefix to all output file names
            '{fileroot}'.format(**self.params)
        ]
        _run( self.params )

        self.params['command_list'] =[
            #tide-search
           '{executable_path}'.format(**self.params),
            'tide-search',
            '{mgf_input_file}'.format(**self.params),
            '{database}_crux_dbindex'.format(**self.params),
            '--overwrite', 'T',    #Replace existing files if true (T) or fail when trying to overwrite a file if false (F)
            # '--txt-output', 'T',#Output an txt fiel results file to the output directory
            '--pin-output', 'T',
            # '--mzid-output', 'T',
            '--concat', 'T',  #report target and decoy hits
            '--parameter-file', #A file containing command-line or additional parameters
            '{param_file}'.format(**self.params),
            '--output-dir', #The name of the directory where output files will be created
            '{output_folder}'.format(**self.params),
            '--fileroot',   #The fileroot string will be added as a prefix to all output file names
            '{fileroot}'.format(**self.params)

        ]


    def postflight( self ):
        '''
        links mzid default name to our specific name

        we  create now tab delimited ouput fr easier parsing and correct conversion
        '''
        pass

        # print('Creating symbolic link for output result...')
        # default_suffix = '.tide-search.mzid'
        # default_file_name = os.path.join(
        #     self.params['output_folder'],
        #     '{0}{1}'.format(
        #         self.params['fileroot'],
        #         default_suffix
        #     )
        # )
        # try:
        #     os.symlink(
        #         default_file_name,
        #         default_file_name.replace(
        #             default_suffix,
        #             '.mzid'
        #         )
        #     )
        # except:
        #     pass




#!/usr/bin/env python
import ursgal
import importlib
import os
import sys
import pprint

class generate_target_decoy_1_0_0( ursgal.UNode ):
    """Generate Target Decoy 1_0_0 UNode"""
    META_INFO = {
        'edit_version'       : 1.00,
        'name'               : 'Generate Target Decoy',
        'version'            : '1.0.0',
        'release_date'       : None,
        'engine_type' : {
            'misc_engine' : True
        },
        'input_extensions'   : [],
        'output_extensions'  : ['.fasta'],
        'output_suffix'      : 'target_decoy',
        'in_development'     : False,
        'include_in_git'     : True,
        'distributable'      : True,
        'utranslation_style' : 'generate_target_decoy_style_1',
        'engine' : {
            'platform_independent' : {
                'arc_independent' : {
                    'exe' : 'generate_target_decoy_1_0_0.py',
                },
            },
        },
        # 'engine_exe'                : {
        #     'arc_independent' : 'generate_target_decoy_1_0_0.py',
        # },
        # 'engine_url' : {
        #     'internal' : True,
        # },
        'citation' : \
            '',
    }

    def __init__(self, *args, **kwargs):
        super(generate_target_decoy_1_0_0, self).__init__(*args, **kwargs)

    def _execute( self ):
        '''
        Creates a target decoy database based on shuffling of peptides or
        complete reversing the protein sequence.

        The engine currently available generates a very stringent target decoy
        database by peptide shuffling but also offers the possibility to
        simple reverse the protein sequence. The mode can be defined in the
        params with 'decoy_generation_mode'.

        The shuffling peptide method is described below.
        As one of the first steps redundant sequences are filtered and the
        protein gets a tag which highlight its double occurence in the database.
        This ensures that no unequal distribution of target and decoy peptides
        is present.
        Further, every peptide is shuffled, while the amindo acids where the
        enzyme cleaves aremaintained at their original position.
        Every peptide is only shuffled once and the shuffling result is stored.
        As a result it is ensured that if a peptide occurs multiple times it is
        shuffled the same way. It is further ensured that unmutable peptides
        (e.g. 'RR' for trypsin) are not shuffled and are reported by the engine
        as unmutable peptides in a text file, so that they can be excluded in
        the further analysis.
        This way of generating a target decoy database lead to the fulfillment
        of the following quality criteria (Proteome Bioinformatics,
        Eds: S.J. Hubbard, A.R. Jones, Humana Press ).

        Quality criteria:

            * every target peptide sequence has exactly one decoy peptide sequence
            * equal amino acid distribution
            * equal protein and peptide length
            * equal number of proteins and peptides
            * similar mass distribution
            * no predicted peptides in common


        Avaliable modes:

            * shuffle_peptide  - stringent target decoy generation with shuffling
                of peptides with maintaining the cleavage site amino acid.

            * reverse_protein - reverses the protein sequence


        Available enzymes and their cleavage site can be found in the knowledge
        base of generate_target_decoy_1_0_0.

        '''

        print('[ -ENGINE- ] Executing conversion ..')
        # self.time_point(tag = 'execution')
        generate_target_decoy_main = self.import_engine_as_python_function()

        input_files = []
        if 'input_file_dicts' in self.params.keys():
            for input_file_dict in self.params['input_file_dicts']:
                input_files.append(
                    os.path.join(
                        input_file_dict['dir'],
                        input_file_dict['file']
                    )
                )
        else:
            input_files.append(
                os.path.join(
                    self.io['input']['finfo']['dir'],
                    self.io['input']['finfo']['file']
                )
            )

        self.params['translations']['output_file_incl_path'] = os.path.join(
            self.params['output_dir_path'],
            self.params['output_file']
        )

        generate_target_decoy_main(
            input_files = input_files,
            output_file = self.params['translations']['output_file_incl_path'],
            enzyme      = self.params['translations']['enzyme'],
            mode        = self.params['translations']['decoy_generation_mode'],
            decoy_tag   = self.params['translations']['decoy_tag'],
            convert_aa_in_motif = self.params['translations']['convert_aa_in_motif'],
        )

        # self.print_execution_time(tag='execution')
        return self.params['output_file_incl_path']



#!/usr/bin/env python3
# encoding: utf-8

import ursgal
import glob
import csv
from collections import defaultdict as ddict
import os
import shutil


def main():
    '''
    Example script to do a simple machine ppm offset parameter sweep.
    The m/z values in the example mgf file are stepwise changed and the in the
    final output the total peptides are counted.

    usage:
        ./bsa_ppm_offset_test.py

    Note:
        As expected, if the offset becomes to big no peptides can be found anymore.

    '''
    ppm_offsets = [
        (-10, '-10_ppm_offset'),
        (-9, '-9_ppm_offset'),
        (-8, '-8_ppm_offset'),
        (-7, '-7_ppm_offset'),
        (-6, '-6_ppm_offset'),
        (-5, '-5_ppm_offset'),
        (-4, '-4_ppm_offset'),
        (-3, '-3_ppm_offset'),
        (-2, '-2_ppm_offset'),
        (-1, '-1_ppm_offset'),
        (None, '0_ppm_offset'),
        (1, '1_ppm_offset'),
        (2, '2_ppm_offset'),
        (3, '3_ppm_offset'),
        (4, '4_ppm_offset'),
        (5, '5_ppm_offset'),
        (6, '6_ppm_offset'),
        (7, '7_ppm_offset'),
        (8, '8_ppm_offset'),
        (9, '9_ppm_offset'),
        (10, '10_ppm_offset'),
    ]

    engine_list = [
        'xtandem_vengeance'
    ]

    R = ursgal.UController(
        profile='LTQ XL low res',
        params={
            'database': os.path.join(
                os.pardir,
                'example_data',
                'BSA.fasta'
            ),
            'modifications': [
                'M,opt,any,Oxidation',        # Met oxidation
                'C,fix,any,Carbamidomethyl',  # Carbamidomethylation
                '*,opt,Prot-N-term,Acetyl'    # N-Acteylation
            ],
        }
    )

    mzML_file = os.path.join(
        os.pardir,
        'example_data',
        'BSA_machine_ppm_offset_example',
        'BSA1.mzML'
    )
    if os.path.exists(mzML_file) is False:
        R.params['http_url'] = 'http://sourceforge.net/p/open-ms/code/HEAD/tree/OpenMS/share/OpenMS/examples/BSA/BSA1.mzML?format=raw'
        R.params['http_output_folder'] = os.path.dirname(mzML_file)
        R.fetch_file(
            engine='get_http_files_1_0_0'
        )
        try:
            shutil.move(
                '{0}?format=raw'.format(mzML_file),
                mzML_file
            )
        except:
            shutil.move(
                '{0}format=raw'.format(mzML_file),
                mzML_file
            )

    for engine in engine_list:
        for (ppm_offset, prefix) in ppm_offsets:

            R.params['machine_offset_in_ppm'] = ppm_offset
            R.params['prefix'] = prefix

            unified_search_result_file = R.search(
                input_file=mzML_file,
                engine=engine,
                force=False,
            )

    collector = ddict(set)
    for csv_path in glob.glob('{0}/*/*unified.csv'.format(os.path.dirname(mzML_file))):
        for line_dict in csv.DictReader(open(csv_path, 'r')):
            collector[csv_path].add(line_dict['Sequence'])
    for csv_path, peptide_set in sorted(collector.items()):
        file_name = os.path.basename(csv_path)
        offset = file_name.split('_')[0]
        print(
            'Search with {0: >3} ppm offset found {1: >2} peptides'.format(
                offset,
                len(peptide_set)
            )
        )

    return

if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# encoding: utf-8

import ursgal
import os


def main():
    '''
    Executes a search with 3 different search engines on an example file from the
    data from Barth et al. (The same file that is used in the XTandem version
    comparison example.)

    usage:
        ./simple_combined_fdr_score.py


    This is a simple example script to show how results from multiple search engines
    can be combined using the Combined FDR Score approach of Jones et al. (2009).
    '''

    engine_list = [
        'omssa_2_1_9',
        'xtandem_piledriver',
        # 'myrimatch_2_1_138',
        'msgfplus_v9979',
    ]

    params = {
        'database': os.path.join(
            os.pardir,
            'example_data',
            'Creinhardtii_281_v5_5_CP_MT_with_contaminants_target_decoy.fasta'
        ),
        'modifications': [],
        'csv_filter_rules': [
            ['PEP', 'lte', 0.01],
            ['Is decoy', 'equals', 'false']
        ],
        'ftp_url': 'ftp.peptideatlas.org',
        'ftp_login': 'PASS00269',
        'ftp_password': 'FI4645a',
        'ftp_include_ext': [
            'JB_FASP_pH8_2-3_28122012.mzML',
        ],
        'ftp_output_folder': os.path.join(
            os.pardir,
            'example_data',
            'xtandem_version_comparison'
        ),
        'http_url': 'https://www.sas.upenn.edu/~sschulze/Creinhardtii_281_v5_5_CP_MT_with_contaminants_target_decoy.fasta',
        'http_output_folder': os.path.join(
            os.pardir,
            'example_data'
        )
    }

    if os.path.exists(params['ftp_output_folder']) is False:
        os.mkdir(params['ftp_output_folder'])

    uc = ursgal.UController(
        profile='LTQ XL low res',
        params=params
    )
    mzML_file = os.path.join(
        params['ftp_output_folder'],
        params['ftp_include_ext'][0]
    )
    if os.path.exists(mzML_file) is False:
        uc.fetch_file(
            engine='get_ftp_files_1_0_0'
        )
    if os.path.exists(params['database']) is False:
        uc.fetch_file(
            engine='get_http_files_1_0_0'
        )

    validated_files_list = []
    for engine in engine_list:

        unified_result_file = uc.search(
            input_file=mzML_file,
            engine=engine,
        )

        validated_file = uc.validate(
            input_file=unified_result_file,
            engine='percolator_2_08',
        )

        validated_files_list.append(validated_file)

    combined_results = uc.combine_search_results(
        input_files=validated_files_list,
        engine='combine_FDR_0_1',
        # use combine_pep_1_0_0 for combined PEP :)
    )
    print('\tCombined results can be found here:')
    print(combined_results)
    return

if __name__ == '__main__':
    main()



#!/usr/bin/env python
import ursgal
import os
import subprocess
from .msgfplus_v2016_09_16 import msgfplus_v2016_09_16 as msgf


class msgfplus_v2019_04_18( msgf ):
    """
    MSGF+ UNode
    Parameter options at https://omics.pnl.gov/software/ms-gf

    Reference:
        Kim S, Mischerikow N, Bandeira N, Navarro JD, Wich L, Mohammed S,
        Heck AJ, Pevzner PA. (2010) The Generating Function of CID, ETD, and
        CID/ETD Pairs of Tandem Mass Spectra: Applications to Database Search.

    Import node for version 2016_09_16

    """
    META_INFO = {
        'edit_version'                : 1.00,
        'name'                        : 'MSGF+',
        'version'                     : 'v2019.04.18',
        'release_date'                : '2019-4-18',
        'engine_type' : {
            'protein_database_search_engine' : True,
        },
        'input_extensions'            : [
            '.mgf',
            '.mzML',
            '.mzXML',
            '.ms2',
            '.pkl',
            '.dta.txt'
        ],
        'output_extensions'           : ['.mzid', '.mzid.gz'],
        'create_own_folder'           : True,
        'in_development'              : False,
        'include_in_git'              : False,
        'utranslation_style'          : 'msgfplus_style_1',
        'distributable'               : True,
        'engine' : {
            'platform_independent' : {
                'arc_independent' : {
                    'exe'            : 'MSGFPlus.jar',
                    'url'            : 'https://github.com/MSGFPlus/msgfplus/releases/tag/v2019.04.18',
                    'zip_md5'        : '539bcfe9426d9fd55dfdd2427a4499c4',
                    'additional_exe' : [],
                },
            },
        },
        'citation' : \
            'Kim S, Mischerikow N, Bandeira N, Navarro JD, Wich L, '\
            'Mohammed S, Heck AJ, Pevzner PA. (2010) The Generating Function '\
            'of CID, ETD, and CID/ETD Pairs of Tandem Mass Spectra: '\
            'Applications to Database Search.',
    }